@@ -2,7 +2,7 @@
  *
  * $Id: code.h,v 1.9 2001/03/19 19:27:39 root Exp $
  *
- * Copyright (C) 1997-2006 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -25,10 +25,11 @@ class CodeOutputInterface;
 class FileDef;
 class MemberDef;
 
-extern void parseCCode(CodeOutputInterface &,const char *,const QCString &, 
+void parseCCode(CodeOutputInterface &,const char *,const QCString &, 
             bool ,const char *,FileDef *fd=0,
             int startLine=-1,int endLine=-1,bool inlineFragment=FALSE,
             MemberDef *memberDef=0);
-extern void resetCCodeParserState();
+void resetCCodeParserState();
+void codeFreeScanner();
 
 #endif
