@@ -3,7 +3,7 @@
  * $Id: dot.h,v 1.14 2001/03/19 19:27:40 root Exp $
  *
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -82,6 +82,7 @@ class DotNode
     int  number() const { return m_number; }
     bool isVisible() const { return m_visible; }
     TruncState isTruncated() const { return m_truncated; }
+    int distance() const { return m_distance; }
 
   private:
     void colorConnectedNodes(int curColor);
@@ -89,6 +90,7 @@ class DotNode
                   bool hasNonReachableChildren, bool reNumber=FALSE);
     void writeArrow(QTextStream &t,GraphType gt,GraphOutputFormat f,DotNode *cn,
                     EdgeInfo *ei,bool topDown, bool pointBack=TRUE, bool reNumber=FALSE);
+    void setDistance(int distance);
     const DotNode   *findDocNode() const; // only works for acyclic graphs!
     void markAsVisible(bool b=TRUE) { m_visible=b; }
     void markAsTruncated(bool b=TRUE) { m_truncated=b ? Truncated : Untruncated; }
@@ -106,6 +108,7 @@ class DotNode
     ClassDef *       m_classDef;  //!< class representing this node (can be 0)
     bool             m_visible;   //!< is the node visible in the output
     TruncState       m_truncated; //!< does the node have non-visible children/parents
+    int              m_distance;  //!< shortest path to the root node
 
     friend class DotGfxHierarchyTable;
     friend class DotClassGraph;
@@ -136,7 +139,7 @@ class DotGfxHierarchyTable
   public:
     DotGfxHierarchyTable();
    ~DotGfxHierarchyTable();
-    void writeGraph(QTextStream &t,const char *path);
+    void writeGraph(QTextStream &t,const char *path) const;
   
   private:
     void addHierarchy(DotNode *n,ClassDef *cd,bool hide);
@@ -155,26 +158,28 @@ class DotClassGraph
     DotClassGraph(ClassDef *cd,DotNode::GraphType t);
    ~DotClassGraph();
     bool isTrivial() const;
+    bool isTooBig() const;
     QCString writeGraph(QTextStream &t,GraphOutputFormat f,const char *path,
-                    const char *relPath, bool TBRank=TRUE,bool imageMap=TRUE);
+                    const char *relPath, bool TBRank=TRUE,bool imageMap=TRUE) const;
 
     void writeXML(QTextStream &t);
     void writeDEF(QTextStream &t);
     QCString diskName() const;
 
   private:
-    void buildGraph(ClassDef *cd,DotNode *n,bool base);
-    void determineVisibleNodes(QList<DotNode> &queue,int &maxNodes,bool includeParents);
+    void buildGraph(ClassDef *cd,DotNode *n,bool base,int distance);
+    bool determineVisibleNodes(DotNode *rootNode,int maxNodes,bool includeParents);
     void determineTruncatedNodes(QList<DotNode> &queue,bool includeParents);
     void addClass(ClassDef *cd,DotNode *n,int prot,const char *label,
                   const char *usedName,const char *templSpec,
-                  bool base);
+                  bool base,int distance);
 
     DotNode        *   m_startNode;
     QDict<DotNode> *   m_usedNodes;
     static int         m_curNodeNumber;
     DotNode::GraphType m_graphType;
     QCString           m_diskName;
+    bool               m_lrRank;
 };
 
 /** @brief Representation of an include dependency graph */
@@ -185,13 +190,14 @@ class DotInclDepGraph
    ~DotInclDepGraph();
     QCString writeGraph(QTextStream &t, GraphOutputFormat f,const char *path,
                     const char *relPath,
-                    bool writeImageMap=TRUE);
+                    bool writeImageMap=TRUE) const;
     bool isTrivial() const;
+    bool isTooBig() const;
     QCString diskName() const;
     void writeXML(QTextStream &t);
 
   private:
-    void buildGraph(DotNode *n,FileDef *fd);
+    void buildGraph(DotNode *n,FileDef *fd,int distance);
     void determineVisibleNodes(QList<DotNode> &queue,int &maxNodes);
     void determineTruncatedNodes(QList<DotNode> &queue);
 
@@ -210,9 +216,10 @@ class DotCallGraph
     DotCallGraph(MemberDef *md,bool inverse);
    ~DotCallGraph();
     QCString writeGraph(QTextStream &t, GraphOutputFormat f,
-                        const char *path,const char *relPath,bool writeImageMap=TRUE);
-    void buildGraph(DotNode *n,MemberDef *md);
+                        const char *path,const char *relPath,bool writeImageMap=TRUE) const;
+    void buildGraph(DotNode *n,MemberDef *md,int distance);
     bool isTrivial() const;
+    bool isTooBig() const;
     void determineVisibleNodes(QList<DotNode> &queue, int &maxNodes);
     void determineTruncatedNodes(QList<DotNode> &queue);
     
@@ -238,7 +245,7 @@ class DotDirDeps
                         GraphOutputFormat format,
                         const char *path,
                         const char *relPath,
-                        bool writeImageMap=TRUE);
+                        bool writeImageMap=TRUE) const;
   private:
     DirDef *m_dir;
 };
@@ -277,20 +284,20 @@ class DotGroupCollaboration
         EdgeType eType;
 
         QList<Link> links;
-        void write( QTextStream &t, int& curNodeId );
+        void write( QTextStream &t ) const;
     };
 
     DotGroupCollaboration(GroupDef* gd);
     ~DotGroupCollaboration();
     QCString writeGraph(QTextStream &t, GraphOutputFormat format,
                     const char *path,const char *relPath,
-                    bool writeImageMap=TRUE);
+                    bool writeImageMap=TRUE) const;
     void buildGraph(GroupDef* gd);
     bool isTrivial() const;
   private :
     void addCollaborationMember( Definition* def, QCString& url, EdgeType eType );
     void addMemberList( class MemberList* ml );
-    void writeGraphHeader(QTextStream &t);
+    void writeGraphHeader(QTextStream &t) const;
     Edge* addEdge( DotNode* _pNStart, DotNode* _pNEnd, EdgeType _eType,
                    const QCString& _label, const QCString& _url );
 
@@ -328,7 +335,8 @@ void generateGraphLegend(const char *path);
 void writeDotGraphFromFile(const char *inFile,const char *outDir,
                            const char *outFile,GraphOutputFormat format);
 QString getDotImageMapFromFile(const QString& inFile, const QString& outDir,
-                               const QCString& relPath);
+                               const QCString& relPath,const QString &context);
 
+void writeDotDirDepGraph(QTextStream &t,DirDef *dd);
 
 #endif
