@@ -2,7 +2,7 @@
  *
  * $Id: groupdef.cpp,v 1.29 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -34,6 +34,10 @@
 #include "docparser.h"
 #include "searchindex.h"
 #include "dot.h"
+#include "vhdldocgen.h"
+#include "layout.h"
+
+//---------------------------------------------------------------------------
 
 GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
                    const char *refFileName) : Definition(df,dl,na)
@@ -59,24 +63,6 @@ GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
   memberGroupSDict->setAutoDelete(TRUE);
 
   allMemberList = new MemberList(MemberList::allMembersList);
-#if 0
-
-  decDefineMembers.setInGroup(TRUE);
-  decProtoMembers.setInGroup(TRUE);
-  decTypedefMembers.setInGroup(TRUE);
-  decEnumMembers.setInGroup(TRUE);
-  decEnumValMembers.setInGroup(TRUE);
-  decFuncMembers.setInGroup(TRUE);
-  decVarMembers.setInGroup(TRUE);
-
-  docDefineMembers.setInGroup(TRUE);
-  docProtoMembers.setInGroup(TRUE);
-  docTypedefMembers.setInGroup(TRUE);
-  docEnumMembers.setInGroup(TRUE);
-  docEnumValMembers.setInGroup(TRUE);
-  docFuncMembers.setInGroup(TRUE);
-  docVarMembers.setInGroup(TRUE);
-#endif
 
   visited = 0;
   groupScope = 0;
@@ -141,14 +127,6 @@ void GroupDef::findSectionsInDocumentation()
       ml->findSectionsInDocumentation();
     }
   }
-#if 0
-  decDefineMembers.findSectionsInDocumentation();
-  decProtoMembers.findSectionsInDocumentation();
-  decTypedefMembers.findSectionsInDocumentation();
-  decEnumMembers.findSectionsInDocumentation();
-  decFuncMembers.findSectionsInDocumentation();
-  decVarMembers.findSectionsInDocumentation();
-#endif
 }
 
 void GroupDef::addFile(const FileDef *def)
@@ -160,22 +138,32 @@ void GroupDef::addFile(const FileDef *def)
     fileList->append(def);
 }
 
-void GroupDef::addClass(const ClassDef *cd)
+bool GroupDef::addClass(const ClassDef *cd)
 {
-  if (cd->isHidden()) return;
-  if (Config_getBool("SORT_BRIEF_DOCS"))
-    classSDict->inSort(cd->name(),cd);
-  else
-    classSDict->append(cd->name(),cd);
+  if (cd->isHidden()) return FALSE;
+  if (classSDict->find(cd->name())==0)
+  {
+    if (Config_getBool("SORT_BRIEF_DOCS"))
+      classSDict->inSort(cd->name(),cd);
+    else
+      classSDict->append(cd->name(),cd);
+    return TRUE;
+  }
+  return FALSE;
 }
 
-void GroupDef::addNamespace(const NamespaceDef *def)
+bool GroupDef::addNamespace(const NamespaceDef *def)
 {
-  if (def->isHidden()) return;
-  if (Config_getBool("SORT_BRIEF_DOCS"))
-    namespaceSDict->inSort(def->name(),def);  
-  else
-    namespaceSDict->append(def->name(),def);
+  if (def->isHidden()) return FALSE;
+  if (namespaceSDict->find(def->name())==0)
+  {
+    if (Config_getBool("SORT_BRIEF_DOCS"))
+      namespaceSDict->inSort(def->name(),def);  
+    else
+      namespaceSDict->append(def->name(),def);
+    return TRUE;
+  }
+  return FALSE;
 }
 
 void GroupDef::addDir(const DirDef *def)
@@ -213,15 +201,6 @@ void GroupDef::addMembersToMemberGroup()
       ::addMembersToMemberGroup(ml,&memberGroupSDict,this);
     }
   }
-#if 0
-  ::addMembersToMemberGroup(&decDefineMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decProtoMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decTypedefMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decEnumMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decEnumValMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decFuncMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decVarMembers,&memberGroupSDict,this);
-#endif
 
   //printf("GroupDef::addMembersToMemberGroup() memberGroupList=%d\n",memberGroupList->count());
   MemberGroupSDict::Iterator mgli(*memberGroupSDict);
@@ -290,107 +269,97 @@ bool GroupDef::insertMember(MemberDef *md,bool docOnly)
       if (!docOnly)
       {
         addMemberToList(MemberList::decVarMembers,md);
-        //if (Config_getBool("SORT_BRIEF_DOCS"))
-        //  decVarMembers.inSort(md);
-        //else
-        //  decVarMembers.append(md);
       }
       addMemberToList(MemberList::docVarMembers,md);
-      //if (Config_getBool("SORT_MEMBER_DOCS"))
-      //  docVarMembers.inSort(md); 
-      //else
-      //  docVarMembers.append(md);
       break;
     case MemberDef::Function: 
       if (!docOnly)
       {
         addMemberToList(MemberList::decFuncMembers,md);
-        //if (Config_getBool("SORT_BRIEF_DOCS"))
-        //  decFuncMembers.inSort(md);
-        //else
-        //  decFuncMembers.append(md);
       }
       addMemberToList(MemberList::docFuncMembers,md);
-      //if (Config_getBool("SORT_MEMBER_DOCS"))    
-      //  docFuncMembers.inSort(md); 
-      //else
-      //  docFuncMembers.append(md);
       break;
     case MemberDef::Typedef:      
       if (!docOnly)
       {
         addMemberToList(MemberList::decTypedefMembers,md);
-        //if (Config_getBool("SORT_BRIEF_DOCS"))
-        //  decTypedefMembers.inSort(md);
-        //else
-        //  decTypedefMembers.append(md);
       }
       addMemberToList(MemberList::docTypedefMembers,md);
-      //if (Config_getBool("SORT_MEMBER_DOCS"))
-      //  docTypedefMembers.inSort(md); 
-      //else
-      //  docTypedefMembers.append(md);
       break;
     case MemberDef::Enumeration:  
       if (!docOnly)
       {
         addMemberToList(MemberList::decEnumMembers,md);
-        //if (Config_getBool("SORT_BRIEF_DOCS"))
-        //  decEnumMembers.inSort(md);
-        //else
-        //  decEnumMembers.append(md);
       }
       addMemberToList(MemberList::docEnumMembers,md);
-      //if (Config_getBool("SORT_MEMBER_DOCS"))
-      //  docEnumMembers.inSort(md); 
-      //else
-      //  docEnumMembers.append(md);
       break;
     case MemberDef::EnumValue:    
       if (!docOnly)
       {
         addMemberToList(MemberList::decEnumValMembers,md);
-        //printf("enum value %s!\n",md->name().data());
-        //if (Config_getBool("SORT_BRIEF_DOCS"))
-        //  decEnumValMembers.inSort(md);
-        //else
-        //  decEnumValMembers.append(md);
       }
       addMemberToList(MemberList::docEnumValMembers,md);
-      //if (Config_getBool("SORT_MEMBER_DOCS"))
-      //  docEnumValMembers.inSort(md); 
-      //else
-      //  docEnumValMembers.append(md);
-      break;
-    case MemberDef::Prototype:    
-      if (!docOnly)
-      {
-        addMemberToList(MemberList::decProtoMembers,md);
-        //if (Config_getBool("SORT_BRIEF_DOCS"))
-        //  decProtoMembers.inSort(md);
-        //else
-        //  decProtoMembers.append(md);
-      }
-      addMemberToList(MemberList::docProtoMembers,md);
-      //if (Config_getBool("SORT_MEMBER_DOCS"))
-      //  docProtoMembers.inSort(md); 
-      //else
-      //  docProtoMembers.append(md);
       break;
     case MemberDef::Define:       
       if (!docOnly)
       {
         addMemberToList(MemberList::decDefineMembers,md);
-        //if (Config_getBool("SORT_BRIEF_DOCS"))
-        //  decDefineMembers.inSort(md);
-        //else
-        //  decDefineMembers.append(md);
       }
       addMemberToList(MemberList::docDefineMembers,md);
-      //if (Config_getBool("SORT_MEMBER_DOCS"))
-      //  docDefineMembers.inSort(md); 
-      //else
-      //  docDefineMembers.append(md);
+      break;
+    case MemberDef::Signal:       
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decSignalMembers,md);
+      }
+      addMemberToList(MemberList::docSignalMembers,md);
+      break;
+    case MemberDef::Slot:       
+      if (md->protection()==Public)
+      {
+        if (!docOnly)
+        {
+          addMemberToList(MemberList::decPubSlotMembers,md);
+        }
+        addMemberToList(MemberList::docPubSlotMembers,md);
+      }
+      else if (md->protection()==Protected)
+      {
+        if (!docOnly)
+        {
+          addMemberToList(MemberList::decProSlotMembers,md);
+        }
+        addMemberToList(MemberList::docProSlotMembers,md);
+      }
+      else
+      {
+        if (!docOnly)
+        {
+          addMemberToList(MemberList::decPriSlotMembers,md);
+        }
+        addMemberToList(MemberList::docPriSlotMembers,md);
+      }
+      break;
+    case MemberDef::Event:       
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decEventMembers,md);
+      }
+      addMemberToList(MemberList::docEventMembers,md);
+      break;
+    case MemberDef::Property:       
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decPropMembers,md);
+      }
+      addMemberToList(MemberList::docPropMembers,md);
+      break;
+    case MemberDef::Friend:       
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decFriendMembers,md);
+      }
+      addMemberToList(MemberList::docFriendMembers,md);
       break;
     default:
       err("GroupDef::insertMembers(): "
@@ -447,14 +416,43 @@ void GroupDef::removeMember(MemberDef *md)
         removeMemberFromList(MemberList::decEnumValMembers,md);
         removeMemberFromList(MemberList::docEnumValMembers,md);
         break;
-      case MemberDef::Prototype:    
-        removeMemberFromList(MemberList::decProtoMembers,md);
-        removeMemberFromList(MemberList::docProtoMembers,md);
-        break;
       case MemberDef::Define:       
         removeMemberFromList(MemberList::decDefineMembers,md);
         removeMemberFromList(MemberList::docDefineMembers,md);
         break;
+      case MemberDef::Signal:       
+        removeMemberFromList(MemberList::decSignalMembers,md);
+        removeMemberFromList(MemberList::docSignalMembers,md);
+        break;
+      case MemberDef::Slot:       
+        if (md->protection()==Public)
+        {
+          removeMemberFromList(MemberList::decPubSlotMembers,md);
+          removeMemberFromList(MemberList::docPubSlotMembers,md);
+        }
+        else if (md->protection()==Protected)
+        {
+          removeMemberFromList(MemberList::decProSlotMembers,md);
+          removeMemberFromList(MemberList::docProSlotMembers,md);
+        }
+        else
+        {
+          removeMemberFromList(MemberList::decPriSlotMembers,md);
+          removeMemberFromList(MemberList::docPriSlotMembers,md);
+        }
+        break;
+      case MemberDef::Event:       
+        removeMemberFromList(MemberList::decEventMembers,md);
+        removeMemberFromList(MemberList::docEventMembers,md);
+        break;
+      case MemberDef::Property:       
+        removeMemberFromList(MemberList::decPropMembers,md);
+        removeMemberFromList(MemberList::docPropMembers,md);
+        break;
+      case MemberDef::Friend:       
+        removeMemberFromList(MemberList::decFriendMembers,md);
+        removeMemberFromList(MemberList::docFriendMembers,md);
+        break;
       default:
         err("GroupDef::removeMember(): unexpected member remove in file!\n");
     }
@@ -499,32 +497,44 @@ void GroupDef::computeAnchors()
   setAnchors(0,'a',allMemberList);
 }
 
-void GroupDef::writeDetailedDocumentation(OutputList &ol)
+void GroupDef::writeDetailedDescription(OutputList &ol,const QCString &title)
 {
-  if (!briefDescription().isEmpty() || !documentation().isEmpty())
+  if ((!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) 
+      || !documentation().isEmpty()
+     )
   {
-    
-    if (pageDict->count()!=countMembers()) // classical layout
+    if (pageDict->count()!=countMembers()) // not only pages -> classical layout
     {
       ol.writeRuler();
       ol.pushGeneratorState();
-      ol.disable(OutputGenerator::Latex);
-      ol.disable(OutputGenerator::RTF);
-      ol.writeAnchor(0,"_details");
+      ol.disableAllBut(OutputGenerator::Html);
+        ol.writeAnchor(0,"_details");
       ol.popGeneratorState();
       ol.startGroupHeader();
-      ol.parseText(theTranslator->trDetailedDescription());
+      ol.parseText(title);
       ol.endGroupHeader();
 
       // repeat brief description
       if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
       {
         ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
-        ol.newParagraph();
+      }
+      // write separator between brief and details
+      if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF") &&
+          !documentation().isEmpty())
+      {
+        ol.pushGeneratorState();
+          ol.disable(OutputGenerator::Man);
+          ol.disable(OutputGenerator::RTF);
+          ol.newParagraph();
+          ol.enableAll();
+          ol.disableAllBut(OutputGenerator::Man);
+          ol.writeString("\n\n");
+        ol.popGeneratorState();
       }
     }
 
-    // write documentation
+    // write detailed documentation
     if (!documentation().isEmpty())
     {
       ol.parseDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE);
@@ -532,29 +542,37 @@ void GroupDef::writeDetailedDocumentation(OutputList &ol)
   }
 }
 
-
-void GroupDef::writeDocumentation(OutputList &ol)
+void GroupDef::writeBriefDescription(OutputList &ol)
 {
-  ol.pushGeneratorState();
-  startFile(ol,getOutputFileBase(),name(),title);
-  startTitle(ol,getOutputFileBase());
-  ol.parseText(title);
-  addGroupListToTitle(ol,this);
-  endTitle(ol,getOutputFileBase(),title);
-
-  if (Config_getBool("SEARCHENGINE"))
+  if (!briefDescription().isEmpty())
   {
-    Doxygen::searchIndex->setCurrentDoc(title,getOutputFileBase());
-    static QRegExp we("[a-zA-Z_][a-zA-Z_0-9]*");
-    int i=0,p=0,l=0;
-    while ((i=we.match(title,p,&l))!=-1) // foreach word in the title
-    {
-      Doxygen::searchIndex->addWord(title.mid(i,l),TRUE);
-      p=i+l;
-    }
-  }
+    ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
+    ol.pushGeneratorState();
+    ol.disable(OutputGenerator::RTF);
+    ol.writeString(" \n");
+    ol.enable(OutputGenerator::RTF);
 
-  if (Config_getBool("HAVE_DOT") && Config_getBool("GROUP_GRAPHS") )
+    if (Config_getBool("REPEAT_BRIEF") ||
+        !documentation().isEmpty()
+       )
+    {
+      ol.disableAllBut(OutputGenerator::Html);
+      ol.startTextLink(0,"_details");
+      ol.parseText(theTranslator->trMore());
+      ol.endTextLink();
+    }
+    ol.popGeneratorState();
+
+    ol.pushGeneratorState();
+    ol.disable(OutputGenerator::RTF);
+    ol.newParagraph();
+    ol.popGeneratorState();
+  }
+}
+
+void GroupDef::writeGroupGraph(OutputList &ol)
+{
+  if (Config_getBool("HAVE_DOT") /*&& Config_getBool("GROUP_GRAPHS")*/ )
   {
     DotGroupCollaboration graph(this);
     if (!graph.isTrivial())
@@ -569,45 +587,15 @@ void GroupDef::writeDocumentation(OutputList &ol)
       ol.popGeneratorState();
     }
   }
+}
 
-
-  if (Config_getBool("DETAILS_AT_TOP"))
-  {
-    writeDetailedDocumentation(ol);
-    ol.newParagraph();
-  }
-  else if (!briefDescription().isEmpty())
-  {
-    ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
-    ol.writeString(" \n");
-    ol.pushGeneratorState();
-    ol.disable(OutputGenerator::Latex);
-    ol.disable(OutputGenerator::RTF);
-    ol.disable(OutputGenerator::Man);
-    ol.startTextLink(0,"_details");
-    ol.parseText(theTranslator->trMore());
-    ol.endTextLink();
-    ol.enableAll();
-    ol.disableAllBut(OutputGenerator::Man);
-    ol.newParagraph();
-    ol.popGeneratorState();
-  }
-
-  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-  {
-    Doxygen::tagFile << "  <compound kind=\"group\">" << endl;
-    Doxygen::tagFile << "    <name>" << convertToXML(name()) << "</name>" << endl;
-    Doxygen::tagFile << "    <title>" << convertToXML(title) << "</title>" << endl;
-    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
-  }
-  
-  ol.startMemberSections();
-
+void GroupDef::writeFiles(OutputList &ol,const QCString &title)
+{
   // write list of files
   if (fileList->count()>0)
   {
     ol.startMemberHeader();
-    ol.parseText(theTranslator->trFile(TRUE,FALSE));
+    ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
     FileDef *fd=fileList->first();
@@ -633,15 +621,21 @@ void GroupDef::writeDocumentation(OutputList &ol)
     }
     ol.endMemberList();
   }
+}
 
+void GroupDef::writeNamespaces(OutputList &ol,const QCString &title)
+{
   // write list of namespaces
-  namespaceSDict->writeDeclaration(ol);
+  namespaceSDict->writeDeclaration(ol,title);
+}
 
+void GroupDef::writeNestedGroups(OutputList &ol,const QCString &title)
+{
   // write list of groups
   if (groupList->count()>0)
   {
     ol.startMemberHeader();
-    ol.parseText(theTranslator->trModules());
+    ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
     GroupDef *gd=groupList->first();
@@ -668,12 +662,15 @@ void GroupDef::writeDocumentation(OutputList &ol)
     }
     ol.endMemberList();
   }
+}
 
+void GroupDef::writeDirs(OutputList &ol,const QCString &title)
+{
   // write list of directories
   if (dirList->count()>0)
   {
     ol.startMemberHeader();
-    ol.parseText(theTranslator->trDirectories());
+    ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
     DirDef *dd=dirList->first();
@@ -700,44 +697,16 @@ void GroupDef::writeDocumentation(OutputList &ol)
 
     ol.endMemberList();
   }
-  
+}
+
+void GroupDef::writeClasses(OutputList &ol,const QCString &title)
+{
   // write list of classes
-  classSDict->writeDeclaration(ol);
-
-  // write list of members
-  if (allMemberList->count()>0)
-  {
-    /* write user defined member groups */
-    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
-    MemberGroup *mg;
-    for (;(mg=mgli.current());++mgli)
-    {
-      mg->writeDeclarations(ol,0,0,0,this);
-    }
-
-    //allMemberList->writeDeclarations(ol,0,0,0,this,0,0); 
-    writeMemberDeclarations(ol,MemberList::decDefineMembers,theTranslator->trDefines());
-    //decDefineMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trDefines(),0);
-    writeMemberDeclarations(ol,MemberList::decProtoMembers,theTranslator->trFuncProtos());
-    //decProtoMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trFuncProtos(),0);
-    writeMemberDeclarations(ol,MemberList::decTypedefMembers,theTranslator->trTypedefs());
-    //decTypedefMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trTypedefs(),0);
-    writeMemberDeclarations(ol,MemberList::decEnumMembers,theTranslator->trEnumerations());
-    //decEnumMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trEnumerations(),0);
-    writeMemberDeclarations(ol,MemberList::decEnumValMembers,theTranslator->trEnumerationValues());
-    //decEnumValMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trEnumerationValues(),0,TRUE);
-    writeMemberDeclarations(ol,MemberList::decFuncMembers,theTranslator->trFunctions());
-    //decFuncMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trFunctions(),0);
-    writeMemberDeclarations(ol,MemberList::decVarMembers,theTranslator->trVariables());
-    //decVarMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trVariables(),0);
-  }
-  ol.endMemberSections();
-
-  if (!Config_getBool("DETAILS_AT_TOP"))
-  {
-    writeDetailedDocumentation(ol);
-  }
+  classSDict->writeDeclaration(ol,0,title,FALSE);
+}
 
+void GroupDef::writePageDocumentation(OutputList &ol)
+{
   PageDef *pd=0;
   PageSDict::Iterator pdi(*pageDict);
   for (pdi.toFirst();(pd=pdi.current());++pdi)
@@ -764,8 +733,279 @@ void GroupDef::writeDocumentation(OutputList &ol)
       ol.endTextBlock();
     }
   }
+}
 
-  writeMemberDocumentation(ol);
+void GroupDef::writeMemberGroups(OutputList &ol)
+{
+  /* write user defined member groups */
+  if (memberGroupSDict)
+  {
+    /* write user defined member groups */
+    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
+    {
+      mg->writeDeclarations(ol,0,0,0,this);
+    }
+  }
+}
+
+void GroupDef::startMemberDeclarations(OutputList &ol)
+{
+  ol.startMemberSections();
+}
+
+void GroupDef::endMemberDeclarations(OutputList &ol)
+{
+  ol.endMemberSections();
+}
+
+void GroupDef::startMemberDocumentation(OutputList &ol)
+{
+  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  {
+    ol.disable(OutputGenerator::Html);
+    Doxygen::suppressDocWarnings = TRUE;
+  }
+}
+
+void GroupDef::endMemberDocumentation(OutputList &ol)
+{
+  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  {
+    ol.enable(OutputGenerator::Html);
+    Doxygen::suppressDocWarnings = FALSE;
+  }
+}
+
+void GroupDef::writeAuthorSection(OutputList &ol)
+{
+  // write Author section (Man only)
+  ol.pushGeneratorState();
+  ol.disableAllBut(OutputGenerator::Man);
+  ol.startGroupHeader();
+  ol.parseText(theTranslator->trAuthor(TRUE,TRUE));
+  ol.endGroupHeader();
+  ol.parseText(theTranslator->trGeneratedAutomatically(Config_getString("PROJECT_NAME")));
+  ol.popGeneratorState();
+}
+
+void GroupDef::writeDocumentation(OutputList &ol)
+{
+  ol.pushGeneratorState();
+  startFile(ol,getOutputFileBase(),name(),title);
+  startTitle(ol,getOutputFileBase());
+  ol.parseText(title);
+  addGroupListToTitle(ol,this);
+  endTitle(ol,getOutputFileBase(),title);
+
+  if (Config_getBool("SEARCHENGINE"))
+  {
+    Doxygen::searchIndex->setCurrentDoc(title,getOutputFileBase());
+    static QRegExp we("[a-zA-Z_][a-zA-Z_0-9]*");
+    int i=0,p=0,l=0;
+    while ((i=we.match(title,p,&l))!=-1) // foreach word in the title
+    {
+      Doxygen::searchIndex->addWord(title.mid(i,l),TRUE);
+      p=i+l;
+    }
+  }
+
+  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
+  {
+    Doxygen::tagFile << "  <compound kind=\"group\">" << endl;
+    Doxygen::tagFile << "    <name>" << convertToXML(name()) << "</name>" << endl;
+    Doxygen::tagFile << "    <title>" << convertToXML(title) << "</title>" << endl;
+    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
+  }
+  
+
+  //---------------------------------------- start flexible part -------------------------------
+
+#define NEW_LAYOUT
+#ifdef NEW_LAYOUT // new flexible layout
+
+  QListIterator<LayoutDocEntry> eli(
+      LayoutDocManager::instance().docEntries(LayoutDocManager::Group));
+  LayoutDocEntry *lde;
+  for (eli.toFirst();(lde=eli.current());++eli)
+  {
+    switch (lde->kind())
+    {
+      case LayoutDocEntry::BriefDesc: 
+        writeBriefDescription(ol);
+        break; 
+      case LayoutDocEntry::MemberDeclStart: 
+        startMemberDeclarations(ol);
+        break; 
+      case LayoutDocEntry::GroupClasses: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeClasses(ol,ls->title);
+        }
+        break; 
+      case LayoutDocEntry::GroupNamespaces: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeNamespaces(ol,ls->title);
+        }
+        break; 
+      case LayoutDocEntry::MemberGroups: 
+        writeMemberGroups(ol);
+        break; 
+      case LayoutDocEntry::MemberDecl: 
+        {
+          LayoutDocEntryMemberDecl *lmd = (LayoutDocEntryMemberDecl*)lde;
+          writeMemberDeclarations(ol,lmd->type,lmd->title);
+        }
+        break; 
+      case LayoutDocEntry::MemberDeclEnd: 
+        endMemberDeclarations(ol);
+        break;
+      case LayoutDocEntry::DetailedDesc: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeDetailedDescription(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::MemberDefStart: 
+        startMemberDocumentation(ol);
+        break; 
+      case LayoutDocEntry::MemberDef: 
+        {
+          LayoutDocEntryMemberDef *lmd = (LayoutDocEntryMemberDef*)lde;
+          writeMemberDocumentation(ol,lmd->type,lmd->title);
+        }
+        break;
+      case LayoutDocEntry::MemberDefEnd: 
+        endMemberDocumentation(ol);
+        break;
+      case LayoutDocEntry::GroupNestedGroups: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeNestedGroups(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::GroupPageDocs: 
+        writePageDocumentation(ol);
+        break;
+      case LayoutDocEntry::GroupDirs: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeDirs(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::GroupFiles: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeFiles(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::GroupGraph: 
+        writeGroupGraph(ol);
+        break;
+      case LayoutDocEntry::AuthorSection: 
+        writeAuthorSection(ol);
+        break;
+      case LayoutDocEntry::ClassIncludes:
+      case LayoutDocEntry::ClassInheritanceGraph:
+      case LayoutDocEntry::ClassNestedClasses:
+      case LayoutDocEntry::ClassCollaborationGraph:
+      case LayoutDocEntry::ClassAllMembersLink:
+      case LayoutDocEntry::ClassUsedFiles:
+      case LayoutDocEntry::NamespaceNestedNamespaces:
+      case LayoutDocEntry::NamespaceClasses:
+      case LayoutDocEntry::FileClasses:
+      case LayoutDocEntry::FileNamespaces:
+      case LayoutDocEntry::FileIncludes:
+      case LayoutDocEntry::FileIncludeGraph:
+      case LayoutDocEntry::FileIncludedByGraph: 
+      case LayoutDocEntry::FileSourceLink:
+      case LayoutDocEntry::DirSubDirs:
+      case LayoutDocEntry::DirFiles:
+      case LayoutDocEntry::DirGraph:
+        err("Internal inconsistency: member %d should not be part of "
+            "LayoutDocManager::Group entry list\n",lde->kind());
+        break;
+    }
+  }
+
+#else
+
+  bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");  
+  bool vhdlOpt    = Config_getBool("OPTIMIZE_OUTPUT_VHDL");  
+
+
+  if (Config_getBool("DETAILS_AT_TOP"))
+  {
+    writeDetailedDescription(ol,theTranslator->trDetailedDescription());
+  }
+  else
+  {
+    writeBriefDescription(ol);
+  }
+
+  writeGroupGraph(ol);
+
+  startMemberDeclarations(ol);
+
+  writeFiles(ol);
+  writeNamespaces(ol);
+  writeNestedGroups(ol);
+  writeDirs(ol);
+  writeClasses(ol);
+  writeMemberGroups(ol);
+
+  writeMemberDeclarations(ol,MemberList::decDefineMembers,theTranslator->trDefines());
+  writeMemberDeclarations(ol,MemberList::decTypedefMembers,theTranslator->trTypedefs());
+  writeMemberDeclarations(ol,MemberList::decEnumMembers,theTranslator->trEnumerations());
+  writeMemberDeclarations(ol,MemberList::decEnumValMembers,theTranslator->trEnumerationValues());
+  writeMemberDeclarations(ol,MemberList::decFuncMembers,
+      fortranOpt ? theTranslator->trSubprograms()  : 
+      vhdlOpt    ? VhdlDocGen::trFunctionAndProc() :
+      theTranslator->trFunctions());
+  writeMemberDeclarations(ol,MemberList::decVarMembers,theTranslator->trVariables());
+  writeMemberDeclarations(ol,MemberList::decSignalMembers,theTranslator->trSignals());
+  writeMemberDeclarations(ol,MemberList::decPubSlotMembers,theTranslator->trPublicSlots());
+  writeMemberDeclarations(ol,MemberList::decProSlotMembers,theTranslator->trProtectedSlots());
+  writeMemberDeclarations(ol,MemberList::decPriSlotMembers,theTranslator->trPrivateSlots());
+  writeMemberDeclarations(ol,MemberList::decEventMembers,theTranslator->trEvents());
+  writeMemberDeclarations(ol,MemberList::decPropMembers,theTranslator->trProperties());
+  writeMemberDeclarations(ol,MemberList::decFriendMembers,theTranslator->trFriends());
+
+  endMemberDeclarations(ol);
+
+  if (!Config_getBool("DETAILS_AT_TOP"))
+  {
+    writeDetailedDescription(ol,theTranslator->trDetailedDescription());
+  }
+
+  startMemberDocumentation(ol);
+  
+  writePageDocumentation(ol);
+
+  writeMemberDocumentation(ol,MemberList::docDefineMembers,theTranslator->trDefineDocumentation());
+  writeMemberDocumentation(ol,MemberList::docTypedefMembers,theTranslator->trTypedefDocumentation());
+  writeMemberDocumentation(ol,MemberList::docEnumMembers,theTranslator->trEnumerationTypeDocumentation());
+  writeMemberDocumentation(ol,MemberList::docEnumValMembers,theTranslator->trEnumerationValueDocumentation());
+  writeMemberDocumentation(ol,MemberList::docFuncMembers,fortranOpt?theTranslator->trSubprogramDocumentation():theTranslator->trFunctionDocumentation());
+  writeMemberDocumentation(ol,MemberList::docVarMembers,theTranslator->trVariableDocumentation());
+  writeMemberDocumentation(ol,MemberList::docSignalMembers,theTranslator->trSignals());   // todo: add trSignalDocumentation()
+  writeMemberDocumentation(ol,MemberList::docPubSlotMembers,theTranslator->trPublicSlots()); // todo: add trSlotDocumentation()
+  writeMemberDocumentation(ol,MemberList::docProSlotMembers,theTranslator->trProtectedSlots()); // todo: add trSlotDocumentation()
+  writeMemberDocumentation(ol,MemberList::docPriSlotMembers,theTranslator->trPrivateSlots()); // todo: add trSlotDocumentation()
+  writeMemberDocumentation(ol,MemberList::docEventMembers,theTranslator->trEvents());     // todo: add trEventDocumentation()
+  writeMemberDocumentation(ol,MemberList::docPropMembers,theTranslator->trProperties());  // todo: add trPropertyDocumentation()
+  writeMemberDocumentation(ol,MemberList::docFriendMembers,theTranslator->trFriends());   // todo: add trFriendDocumentation()
+
+  endMemberDocumentation(ol);
+
+#endif
+
+  //---------------------------------------- end flexible part -------------------------------
+
+  endFile(ol); 
+  ol.popGeneratorState();
 
   if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
   {
@@ -773,9 +1013,6 @@ void GroupDef::writeDocumentation(OutputList &ol)
     Doxygen::tagFile << "  </compound>" << endl;
   }
 
-  endFile(ol); 
-  ol.popGeneratorState();
-
   if (Config_getBool("SEPARATE_MEMBER_PAGES"))
   {
     allMemberList->sort();
@@ -784,47 +1021,6 @@ void GroupDef::writeDocumentation(OutputList &ol)
 
 }
 
-void GroupDef::writeMemberDocumentation(OutputList &ol)
-{
-  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
-  {
-    ol.disable(OutputGenerator::Html);
-  }
-  
-  writeMemberDocumentation(ol,MemberList::docDefineMembers,theTranslator->trDefineDocumentation());
-  //docDefineMembers.writeDocumentation(ol,name(),this,
-  //                           theTranslator->trDefineDocumentation());
-  
-  writeMemberDocumentation(ol,MemberList::docProtoMembers,theTranslator->trFunctionPrototypeDocumentation());
-  //docProtoMembers.writeDocumentation(ol,name(),this,
-  //                           theTranslator->trFunctionPrototypeDocumentation());
-
-  writeMemberDocumentation(ol,MemberList::docTypedefMembers,theTranslator->trTypedefDocumentation());
-  //docTypedefMembers.writeDocumentation(ol,name(),this,
-  //                           theTranslator->trTypedefDocumentation());
-  
-  writeMemberDocumentation(ol,MemberList::docEnumMembers,theTranslator->trEnumerationTypeDocumentation());
-  //docEnumMembers.writeDocumentation(ol,name(),this,
-  //                           theTranslator->trEnumerationTypeDocumentation());
-
-  writeMemberDocumentation(ol,MemberList::docEnumValMembers,theTranslator->trEnumerationValueDocumentation());
-  //docEnumValMembers.writeDocumentation(ol,name(),this,
-  //                           theTranslator->trEnumerationValueDocumentation(),TRUE);
-
-  writeMemberDocumentation(ol,MemberList::docFuncMembers,theTranslator->trFunctionDocumentation());
-  //docFuncMembers.writeDocumentation(ol,name(),this,
-  //                           theTranslator->trFunctionDocumentation());
-  
-  writeMemberDocumentation(ol,MemberList::docVarMembers,theTranslator->trVariableDocumentation());
-  //docVarMembers.writeDocumentation(ol,name(),this,
-  //                           theTranslator->trVariableDocumentation());
-
-  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
-  {
-    ol.enable(OutputGenerator::Html);
-  }
-}
-
 void GroupDef::writeMemberPages(OutputList &ol)
 {
   ol.pushGeneratorState();
@@ -839,14 +1035,6 @@ void GroupDef::writeMemberPages(OutputList &ol)
        ml->writeDocumentationPage(ol,name(),this);
     }
   }
-#if 0
-  docDefineMembers.writeDocumentationPage(ol,name(),this);
-  docProtoMembers.writeDocumentationPage(ol,name(),this);
-  docTypedefMembers.writeDocumentationPage(ol,name(),this);
-  docEnumMembers.writeDocumentationPage(ol,name(),this);
-  docFuncMembers.writeDocumentationPage(ol,name(),this);
-  docVarMembers.writeDocumentationPage(ol,name(),this);
-#endif
 
   ol.popGeneratorState();
 }
@@ -903,8 +1091,7 @@ void addClassToGroups(Entry *root,ClassDef *cd)
     GroupDef *gd=0;
     if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g->groupname)))
     {
-      gd->addClass(cd);
-      cd->makePartOfGroup(gd);
+      if (gd->addClass(cd)) cd->makePartOfGroup(gd);
       //printf("Compound %s: in group %s\n",cd->name().data(),s->data());
     }
   }
@@ -921,8 +1108,7 @@ void addNamespaceToGroups(Entry *root,NamespaceDef *nd)
     //printf("group `%s'\n",s->data());
     if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g->groupname)))
     {
-      gd->addNamespace(nd);
-      nd->makePartOfGroup(gd);
+      if (gd->addNamespace(nd)) nd->makePartOfGroup(gd);
       //printf("Namespace %s: in group %s\n",nd->name().data(),s->data());
     }
   }
@@ -1123,14 +1309,6 @@ void GroupDef::addListReferences()
       ml->addListReferences(this);
     }
   }
-#if 0
-  docDefineMembers.addListReferences(this);
-  docProtoMembers.addListReferences(this);
-  docTypedefMembers.addListReferences(this);
-  docEnumMembers.addListReferences(this);
-  docFuncMembers.addListReferences(this);
-  docVarMembers.addListReferences(this);
-#endif
 }
 
 MemberList *GroupDef::createMemberList(MemberList::ListType lt)
@@ -1182,8 +1360,16 @@ MemberList *GroupDef::getMemberList(MemberList::ListType lt) const
 
 void GroupDef::writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title)
 {
+   static bool optimizeVhdl = Config_getBool("OPTIMIZE_OUTPUT_VHDL");
+ 
   MemberList * ml = getMemberList(lt);
-  if (ml) ml->writeDeclarations(ol,0,0,0,this,title,0);
+    if (optimizeVhdl && ml) 
+    {
+         VhdlDocGen::writeVhdlDeclarations(ml,ol,this,0,0);
+        return;
+    }
+  if (ml) 
+  ml->writeDeclarations(ol,0,0,0,this,title,0);
 }
 
 void GroupDef::writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title)
