@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -28,6 +28,7 @@
 #include "htmlgen.h"
 #include "parserintf.h"
 #include "msc.h"
+#include "util.h"
 
 
 static const int NUM_HTML_LIST_TYPES = 4;
@@ -243,7 +244,7 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
         file.close();
 
         m_t << "<div align=\"center\">" << endl;
-        writeDotFile(fileName,s->relPath());
+        writeDotFile(fileName,s->relPath(),s->context());
         m_t << "</div>" << endl;
 
         if (Config_getBool("DOT_CLEANUP")) file.remove();
@@ -270,7 +271,7 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
         file.close();
 
         m_t << "<div align=\"center\">" << endl;
-        writeMscFile(baseName,s->relPath());
+        writeMscFile(baseName,s->relPath(),s->context());
         m_t << "</div>" << endl;
 
         if (Config_getBool("DOT_CLEANUP")) file.remove();
@@ -293,8 +294,16 @@ void HtmlDocVisitor::visit(DocInclude *inc)
     case DocInclude::Include: 
       m_t << PREFRAG_START;
       Doxygen::parserManager->getParser(inc->extension())
-                            ->parseCode(m_ci,inc->context(),inc->text().latin1(),
-                                        inc->isExample(),inc->exampleFile());
+                            ->parseCode(m_ci,                 
+                                        inc->context(),
+                                        inc->text().latin1(),
+                                        inc->isExample(),
+                                        inc->exampleFile(),
+                                        0,   // fd
+                                        -1,  // startLine
+                                        -1,  // endLine
+                                        TRUE // inlineFragment
+                                       );
       m_t << PREFRAG_END;
       break;
     case DocInclude::IncWithLines:
@@ -303,7 +312,8 @@ void HtmlDocVisitor::visit(DocInclude *inc)
          QFileInfo cfi( inc->file() );
          FileDef fd( cfi.dirPath(), cfi.fileName() );
          Doxygen::parserManager->getParser(inc->extension())
-                               ->parseCode(m_ci,inc->context(),
+                               ->parseCode(m_ci,
+                                           inc->context(),
                                            inc->text().latin1(),
                                            inc->isExample(),
                                            inc->exampleFile(), &fd);
@@ -565,7 +575,7 @@ void HtmlDocVisitor::visitPre(DocSection *s)
   m_t << "<h" << s->level()+1 << ">";
   m_t << "<a class=\"anchor\" name=\"" << s->anchor();
   m_t << "\">" << endl;
-  filter(s->title());
+  filter(convertCharEntitiesToUTF8(s->title().data()));
   m_t << "</a></h" << s->level()+1 << ">\n";
 }
 
@@ -577,18 +587,26 @@ void HtmlDocVisitor::visitPre(DocHtmlList *s)
 {
   if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
+  {
     m_t << "<ol" << htmlAttribsToString(s->attribs()) << ">\n"; 
+  }
   else 
+  {
     m_t << "<ul" << htmlAttribsToString(s->attribs()) << ">\n";
+  }
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlList *s) 
 {
   if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
+  {
     m_t << "</ol>"; 
-  else 
+  }
+  else
+  { 
     m_t << "</ul>";
+  }
   if (!s->isPreformatted()) m_t << "\n";
 }
 
@@ -806,7 +824,7 @@ void HtmlDocVisitor::visitPost(DocImage *img)
 void HtmlDocVisitor::visitPre(DocDotFile *df)
 {
   if (m_hide) return;
-  writeDotFile(df->file(),df->relPath());
+  writeDotFile(df->file(),df->relPath(),df->context());
   m_t << "<div align=\"center\">" << endl;
   if (df->hasCaption())
   { 
@@ -917,6 +935,11 @@ void HtmlDocVisitor::visitPre(DocParamSect *s)
       m_t << theTranslator->trReturnValues(); break;
     case DocParamSect::Exception: 
       m_t << theTranslator->trExceptions(); break;
+    case DocParamSect::TemplateParam: 
+      /* TODO: add this 
+      m_t << theTranslator->trTemplateParam(); break;
+      */
+      m_t << "Template Parameters"; break;
     default:
       ASSERT(0);
   }
@@ -1140,7 +1163,8 @@ void HtmlDocVisitor::popEnabled()
   delete v;
 }
 
-void HtmlDocVisitor::writeDotFile(const QString &fileName,const QString &relPath)
+void HtmlDocVisitor::writeDotFile(const QString &fileName,const QString &relPath,
+                                  const QString &context)
 {
   QString baseName=fileName;
   int i;
@@ -1155,11 +1179,12 @@ void HtmlDocVisitor::writeDotFile(const QString &fileName,const QString &relPath
   m_t << "<img src=\"" << relPath << baseName << "." 
     << Config_getEnum("DOT_IMAGE_FORMAT") << "\" alt=\""
     << baseName << "\" border=\"0\" usemap=\"#" << mapName << "\">" << endl;
-  QString imap = getDotImageMapFromFile(fileName,outDir,relPath.data());
+  QString imap = getDotImageMapFromFile(fileName,outDir,relPath.data(),context);
   m_t << "<map name=\"" << mapName << "\">" << imap << "</map>" << endl;
 }
 
-void HtmlDocVisitor::writeMscFile(const QString &fileName,const QString &relPath)
+void HtmlDocVisitor::writeMscFile(const QString &fileName,const QString &relPath,
+                                  const QString &context)
 {
   QString baseName=fileName;
   int i;
@@ -1173,7 +1198,7 @@ void HtmlDocVisitor::writeMscFile(const QString &fileName,const QString &relPath
   QString mapFile = fileName+".map";
   m_t << "<img src=\"" << relPath << baseName << ".png\" alt=\""
     << baseName << "\" border=\"0\" usemap=\"#" << mapName << "\">" << endl;
-  QString imap = getMscImageMapFromFile(fileName,outDir,relPath.data());
+  QString imap = getMscImageMapFromFile(fileName,outDir,relPath.data(),context);
   m_t << "<map name=\"" << mapName << "\">" << imap << "</map>" << endl;
 }
 
