@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.cpp,v 1.56 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -36,6 +36,8 @@
 #include "pagedef.h"
 #include "debug.h"
 #include "dirdef.h"
+#include "vhdldocgen.h"
+#include "layout.h"
 
 // #define GROUP_COLOR "#ff8080"
 
@@ -46,7 +48,7 @@ static const char defaultStyleSheet[] =
 #include "doxygen_css.h"
 ;
 
-const char search_script[]=
+static const char search_script[]=
 #include "search_php.h"
 ;
 
@@ -443,7 +445,7 @@ static const char tabs_css[] =
 "   padding-bottom   : 6px;\n"
 "}\n"
 "\n"
-"DIV.nav\n"
+"DIV.navpath\n"
 "{\n"
 "   background       : none;\n"
 "   border           : none;\n"
@@ -488,7 +490,29 @@ static void writeTabData(const char *dir)
   }
 }
 
-//-----------------------------------------------------------------------
+//------------------------------------------------------------------------
+
+unsigned char open_gif[] = {
+  0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 0x09, 0x00, 0x09, 0x00, 0xf0, 0x00,
+  0x00, 0x8e, 0xaf, 0xc4, 0x00, 0x00, 0x00, 0x21, 0xf9, 0x04, 0x01, 0x00,
+  0x00, 0x01, 0x00, 0x2c, 0x00, 0x00, 0x00, 0x00, 0x09, 0x00, 0x09, 0x00,
+  0x00, 0x02, 0x0d, 0x8c, 0x8f, 0xa9, 0xcb, 0xe0, 0xff, 0x02, 0x8c, 0x66,
+  0x26, 0x7a, 0x51, 0x01, 0x00, 0x3b
+};
+unsigned int open_gif_len = 54;
+
+unsigned char closed_gif[] = {
+  0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 0x09, 0x00, 0x09, 0x00, 0xf0, 0x00,
+  0x00, 0x8e, 0xaf, 0xc4, 0x00, 0x00, 0x00, 0x21, 0xf9, 0x04, 0x01, 0x00,
+  0x00, 0x01, 0x00, 0x2c, 0x00, 0x00, 0x00, 0x00, 0x09, 0x00, 0x09, 0x00,
+  0x00, 0x02, 0x10, 0x8c, 0x03, 0xa7, 0x98, 0xcb, 0xad, 0x80, 0x84, 0x66,
+  0xca, 0x38, 0x57, 0xd6, 0xf4, 0xd0, 0x02, 0x00, 0x3b
+};
+unsigned int closed_gif_len = 57;
+
+
+
+//------------------------- Pictures for the Tabs ------------------------
 
 HtmlGenerator::HtmlGenerator() : OutputGenerator()
 {
@@ -578,9 +602,7 @@ static void writeDefaultHeaderFile(QTextStream &t, const char *title,
 void HtmlGenerator::writeHeaderFile(QFile &file)
 {
   QTextStream t(&file);
-#if QT_VERSION >= 200
-  t.setEncoding(QTextStream::Latin1);
-#endif
+  t.setEncoding(QTextStream::UnicodeUTF8);
   writeDefaultHeaderFile(t,"$title",relativePathToRoot(0),TRUE);
 }
 
@@ -598,6 +620,55 @@ void HtmlGenerator::writeFooterFile(QFile &file)
     << "</html>\n";
 }
 
+static void generateDynamicSections(QTextStream &t,const QCString &relPath)
+{
+  if (Config_getBool("HTML_DYNAMIC_SECTIONS"))
+  { 
+    t << 
+      "<script type=\"text/javascript\">\n"
+      "<!--\n"
+      "function changeDisplayState (e){\n"
+      "  var num=this.id.replace(/[^[0-9]/g,'');\n"
+      "  var button=this.firstChild;\n"
+      "  var sectionDiv=document.getElementById('dynsection'+num);\n"
+      "  if (sectionDiv.style.display=='none'||sectionDiv.style.display==''){\n"
+      "    sectionDiv.style.display='block';\n"
+      "    button.src='" << relPath << "open.gif';\n"
+      "  }else{\n"
+      "    sectionDiv.style.display='none';\n"
+      "    button.src='" << relPath << "closed.gif';\n"
+      "  }\n"
+      "}\n"
+      "function initDynSections(){\n"
+      "  var divs=document.getElementsByTagName('div');\n"
+      "  var sectionCounter=1;\n"
+      "  for(var i=0;i<divs.length-1;i++){\n"
+      "    if(divs[i].className=='dynheader'&&divs[i+1].className=='dynsection'){\n"
+      "      var header=divs[i];\n"
+      "      var section=divs[i+1];\n"
+      "      var button=header.firstChild;\n"
+      "      if (button!='IMG'){\n"
+      "        divs[i].insertBefore(document.createTextNode(' '),divs[i].firstChild);\n"
+      "        button=document.createElement('img');\n"
+      "        divs[i].insertBefore(button,divs[i].firstChild);\n"
+      "      }\n"
+      "      header.style.cursor='pointer';\n"
+      "      header.onclick=changeDisplayState;\n"
+      "      header.id='dynheader'+sectionCounter;\n"
+      "      button.src='" << relPath << "closed.gif';\n"
+      "      section.id='dynsection'+sectionCounter;\n"
+      "      section.style.display='none';\n"
+      "      section.style.marginLeft='14px';\n"
+      "      sectionCounter++;\n"
+      "    }\n"
+      "  }\n"
+      "}\n"
+      "window.onload = initDynSections;\n"
+      "-->\n"
+      "</script>\n";
+  }
+}
+
 
 void HtmlGenerator::startFile(const char *name,const char *,
                               const char *title)
@@ -612,10 +683,11 @@ void HtmlGenerator::startFile(const char *name,const char *,
     fileName+=Doxygen::htmlFileExtension;
   }
   startPlainFile(fileName);
-  if (Config_getBool("GENERATE_HTMLHELP"))
-  {
-    HtmlHelp::getInstance()->addIndexFile(fileName);
-  }
+  Doxygen::indexList.addIndexFile(fileName);
+  //if (Config_getBool("GENERATE_HTMLHELP"))
+  //{
+  //  HtmlHelp::getInstance()->addIndexFile(fileName);
+  //}
   
   QCString dispTitle = title;
   QCString projName = Config_getString("PROJECT_NAME");
@@ -635,9 +707,9 @@ void HtmlGenerator::startFile(const char *name,const char *,
   }
   t << "<!-- " << theTranslator->trGeneratedBy() << " Doxygen " 
     << versionString << " -->" << endl;
+  generateDynamicSections(t,relPath);
 }
 
-
 static void writePageFooter(QTextStream &t,const QCString &lastTitle,
                             const QCString relPath)
 {
@@ -734,11 +806,11 @@ void HtmlGenerator::startDoxyAnchor(const char *,const char *,
 {
   t << "<a class=\"anchor\" name=\"" << anchor << "\"></a>";
   t << "<!-- doxytag: member=\"";
-  docify(name); 
+  docify(name,TRUE); 
   t << "\" ref=\""; 
-  docify(anchor); 
+  docify(anchor,TRUE); 
   t << "\" args=\"";
-  docify(args);
+  docify(args,TRUE);
   t << "\" -->";
 }
 
@@ -751,6 +823,16 @@ void HtmlGenerator::newParagraph()
   t << endl << "<p>" << endl;
 }
 
+void HtmlGenerator::startParagraph()
+{
+  t << endl << "<p>";
+}
+
+void HtmlGenerator::endParagraph()
+{
+  t << "</p>" << endl;
+}
+
 void HtmlGenerator::writeString(const char *text)
 {
   t << text;
@@ -758,7 +840,7 @@ void HtmlGenerator::writeString(const char *text)
 
 void HtmlGenerator::startIndexItem(const char *ref,const char *f)
 {
-  //printf("HtmlGenerator::startIndexItem(%s,%s,%s)\n",ref,f,name);
+  //printf("HtmlGenerator::startIndexItem(%s,%s)\n",ref,f);
   QCString *dest;
   t << "<li>";
   if (ref || f)
@@ -850,6 +932,7 @@ void HtmlGenerator::writeCodeLink(const char *ref,const char *f,
                                   const char *tooltip)
 {
   QCString *dest;
+  //printf("writeCodeLink(ref=%s,f=%s,anchor=%s,name=%s,tooltip=%s)\n",ref,f,anchor,name,tooltip);
   if (ref) 
   {
     t << "<a class=\"codeRef\" ";
@@ -873,7 +956,7 @@ void HtmlGenerator::writeCodeLink(const char *ref,const char *f,
   if (f) t << f << Doxygen::htmlFileExtension;
   if (anchor) t << "#" << anchor;
   t << "\"";
-  if (tooltip) t << " title=\"" << tooltip << "\"";
+  if (tooltip) t << " title=\"" << convertToXML(tooltip) << "\"";
   t << ">";
   docify(name);
   t << "</a>";
@@ -896,7 +979,7 @@ void HtmlGenerator::endTextLink()
 void HtmlGenerator::startHtmlLink(const char *url)
 {
   t << "<a ";
-  if (Config_getBool("GENERATE_TREEVIEW")) t << "target=\"top\" ";
+  if (usingTreeIndex()) t << "target=\"top\" ";
   t << "href=\"";
   if (url) t << url;
   t << "\">"; 
@@ -946,6 +1029,11 @@ void HtmlGenerator::endSection(const char *,SectionInfo::SectionType type)
 }
 
 void HtmlGenerator::docify(const char *str)
+{
+  docify(str,FALSE);
+}
+
+void HtmlGenerator::docify(const char *str,bool inHtmlComment)
 {
   if (str)
   {
@@ -960,6 +1048,7 @@ void HtmlGenerator::docify(const char *str)
         case '>':  t << "&gt;"; break;
         case '&':  t << "&amp;"; break;
         case '"':  t << "&quot;"; break;
+        case '-':  if (inHtmlComment) t << "&#45;"; else t << "-"; break;
         case '\\':
                    if (*p=='<')
                      { t << "&lt;"; p++; }
@@ -1033,18 +1122,22 @@ void HtmlGenerator::writeChar(char c)
 
 void HtmlGenerator::startClassDiagram()
 {
-  t << "<p>";
+  //t << "<p>";
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endClassDiagram(ClassDiagram &d,
+void HtmlGenerator::endClassDiagram(const ClassDiagram &d,
                                 const char *fileName,const char *name)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   t << "\n<p><center><img src=\""
     << relPath << fileName << ".png\" usemap=\"#" << name << "_map\""
     << " border=\"0\" alt=\"\"></center>" << endl
     << "<map name=\"" << name << "_map\">" << endl;
 
   d.writeImage(t,dir,relPath,fileName);
+  t << "</div>" << endl;
 }
 
 
@@ -1358,7 +1451,7 @@ void HtmlGenerator::endParameterName(bool last,bool emptyList,bool closeBracket)
       t << "          <td>";
       if (closeBracket) t << "&nbsp;)";
       t << "&nbsp;</td>" << endl;
-      t << "          <td width=\"100%\">";
+      t << "          <td>";
     }
     else
     {
@@ -1369,7 +1462,7 @@ void HtmlGenerator::endParameterName(bool last,bool emptyList,bool closeBracket)
       t << "          <td>";
       if (closeBracket) t << ")";
       t << "</td>" << endl;
-      t << "          <td></td><td></td><td width=\"100%\">";
+      t << "          <td></td><td></td><td>";
     }
   }
   else
@@ -1402,52 +1495,78 @@ void HtmlGenerator::endMemberDoc(bool hasArgs)
 
 void HtmlGenerator::startDotGraph()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endDotGraph(DotClassGraph &g)
+void HtmlGenerator::endDotGraph(const DotClassGraph &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  if (Config_getBool("GENERATE_LEGEND"))
+  {
+    t << "<center><font size=\"2\">[";
+    startHtmlLink(relPath+"graph_legend"+Doxygen::htmlFileExtension);
+    t << theTranslator->trLegend();
+    endHtmlLink();
+    t << "]</font></center>";
+  }
+  t << "</div>" << endl;
 }
 
 void HtmlGenerator::startInclDepGraph()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endInclDepGraph(DotInclDepGraph &g)
+void HtmlGenerator::endInclDepGraph(const DotInclDepGraph &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  t << "</div>" << endl;
 }
 
-
 void HtmlGenerator::startGroupCollaboration()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endGroupCollaboration(DotGroupCollaboration &g)
+void HtmlGenerator::endGroupCollaboration(const DotGroupCollaboration &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  t << "</div>" << endl;
 }
 
-
 void HtmlGenerator::startCallGraph()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endCallGraph(DotCallGraph &g)
+void HtmlGenerator::endCallGraph(const DotCallGraph &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  t << "</div>" << endl;
 }
 
 void HtmlGenerator::startDirDepGraph()
 {
+  t << "<div class=\"dynheader\">" << endl;
 }
 
-void HtmlGenerator::endDirDepGraph(DotDirDeps &g)
+void HtmlGenerator::endDirDepGraph(const DotDirDeps &g)
 {
+  t << "</div>" << endl;
+  t << "<div class=\"dynsection\">" << endl;
   g.writeGraph(t,BITMAP,dir,relPath);
+  t << "</div>" << endl;
 }
 
-void HtmlGenerator::writeGraphicalHierarchy(DotGfxHierarchyTable &g)
+void HtmlGenerator::writeGraphicalHierarchy(const DotGfxHierarchyTable &g)
 {
   g.writeGraph(t,dir);
 }
@@ -1506,17 +1625,17 @@ void HtmlGenerator::writeNonBreakableSpace(int n)
   }
 }
 
-void HtmlGenerator::writeLineNumber(const char *ref,const char *file,
+void HtmlGenerator::writeLineNumber(const char *ref,const char *filename,
                                     const char *anchor,int l)
 {
   QCString lineNumber,lineAnchor;
   lineNumber.sprintf("%05d",l);
   lineAnchor.sprintf("l%05d",l);
 
-  if (file)
+  if (filename)
   {
     startCodeAnchor(lineAnchor);
-    writeCodeLink(ref,file,anchor,lineNumber,0);
+    writeCodeLink(ref,filename,anchor,lineNumber,0);
     endCodeAnchor();
   }
   else
@@ -1529,13 +1648,13 @@ void HtmlGenerator::writeLineNumber(const char *ref,const char *file,
 }
 
 void HtmlGenerator::startSimpleSect(SectionTypes,
-                                const char *file,const char *anchor,
+                                const char *filename,const char *anchor,
                                 const char *title)
 {
   t << "<dl compact><dt><b>";
-  if (file)
+  if (filename)
   {
-    writeObjectLink(0,file,anchor,title);
+    writeObjectLink(0,filename,anchor,title);
   }
   else
   {
@@ -1575,8 +1694,8 @@ static void startQuickIndexList(QTextStream &t,bool compact)
 {
   if (compact) 
   {
-    t << "<div class=\"tabs\">\n";
-    t << "  <ul>\n"; 
+    t << "  <div class=\"tabs\">\n";
+    t << "    <ul>\n"; 
   }
   else 
   {
@@ -1588,8 +1707,8 @@ static void endQuickIndexList(QTextStream &t,bool compact)
 {
   if (compact) 
   {
-    t << "  </ul>\n";
-    t << "</div>\n";
+    t << "    </ul>\n";
+    t << "  </div>\n";
   }
   else 
   {
@@ -1601,7 +1720,7 @@ static void startQuickIndexItem(QTextStream &t,const char *l,
                                 bool hl,bool /*compact*/,
                                 const QCString &relPath)
 {
-  t << "    <li"; if (hl) t << " class=\"current\"";
+  t << "      <li"; if (hl) t << " class=\"current\"";
   t << "><a ";
   t << "href=\"" << relPath << l << "\">";
   t << "<span>";
@@ -1619,10 +1738,183 @@ static QCString fixSpaces(const QCString &s)
   return substitute(s," ","&nbsp;");
 }
 
+#define NEW_LAYOUT
+#ifdef NEW_LAYOUT
+
+static bool quickLinkVisible(LayoutNavEntry::Kind kind)
+{
+  switch (kind)
+  {
+    case LayoutNavEntry::MainPage:         return TRUE; 
+    case LayoutNavEntry::Pages:            return indexedPages>0;
+    case LayoutNavEntry::Modules:          return documentedGroups>0;
+    case LayoutNavEntry::Namespaces:       return documentedNamespaces>0;
+    case LayoutNavEntry::NamespaceMembers: return documentedNamespaceMembers[NMHL_All]>0;
+    case LayoutNavEntry::Classes:          return annotatedClasses>0;
+    case LayoutNavEntry::ClassAnnotated:   return annotatedClasses>0; 
+    case LayoutNavEntry::ClassHierarchy:   return hierarchyClasses>0;
+    case LayoutNavEntry::ClassMembers:     return documentedClassMembers[CMHL_All]>0;
+    case LayoutNavEntry::Files:            return documentedHtmlFiles>0;
+    case LayoutNavEntry::FileGlobals:      return documentedFileMembers[FMHL_All]>0;
+    case LayoutNavEntry::Dirs:             return documentedDirs>0;
+    case LayoutNavEntry::Examples:         return Doxygen::exampleSDict->count()>0;
+  }
+  return FALSE;
+}
+
+static void renderQuickLinksAsTree(QTextStream &t,const QCString &relPath,LayoutNavEntry *root)
+
+{
+  QListIterator<LayoutNavEntry> li(root->children());
+  LayoutNavEntry *entry;
+  int count=0;
+  for (li.toFirst();(entry=li.current());++li)
+  {
+    if (entry->visible() && quickLinkVisible(entry->kind())) count++;
+  }
+  if (count>0) // at least one item is visible
+  {
+    startQuickIndexList(t,FALSE);
+    for (li.toFirst();(entry=li.current());++li)
+    {
+      if (entry->visible() && quickLinkVisible(entry->kind()))
+      {
+        startQuickIndexItem(t,entry->baseFile()+Doxygen::htmlFileExtension,
+                            FALSE,FALSE,relPath);
+        t << fixSpaces(entry->title());
+        endQuickIndexItem(t);
+        // recursive into child list
+        renderQuickLinksAsTree(t,relPath,entry);
+      }
+    }
+    endQuickIndexList(t,FALSE);
+  }
+}
+
+
+static void renderQuickLinksAsTabs(QTextStream &t,const QCString &relPath,
+                             LayoutNavEntry *hlEntry,LayoutNavEntry::Kind kind,
+                             bool highlightParent,bool highlightSearch)
+{
+  if (hlEntry->parent()) // first draw the tabs for the parent of hlEntry
+  {
+    renderQuickLinksAsTabs(t,relPath,hlEntry->parent(),kind,highlightParent,highlightSearch);
+  }
+  if (hlEntry->parent() && hlEntry->parent()->children().count()>0) // draw tabs for row containing hlEntry
+  {
+    QListIterator<LayoutNavEntry> li(hlEntry->parent()->children());
+    LayoutNavEntry *entry;
+
+    int count=0;
+    for (li.toFirst();(entry=li.current());++li)
+    {
+      if (entry->visible() && quickLinkVisible(entry->kind())) count++;
+    }
+    if (count>0) // at least one item is visible
+    {
+      startQuickIndexList(t,TRUE);
+      for (li.toFirst();(entry=li.current());++li)
+      {
+        if (entry->visible() && quickLinkVisible(entry->kind()))
+        {
+          startQuickIndexItem(t,entry->baseFile()+Doxygen::htmlFileExtension,
+              entry==hlEntry  && (entry->children().count()>0 || (entry->kind()==kind && !highlightParent)),
+              TRUE,relPath);
+          t << fixSpaces(entry->title());
+          endQuickIndexItem(t);
+        }
+      }
+      if (hlEntry->parent()==LayoutDocManager::instance().rootNavEntry())
+      {
+        // last item of the top row -> special case for search engine
+        if (Config_getBool("SEARCHENGINE"))
+        {
+          QCString searchFor = fixSpaces(theTranslator->trSearchForIndex());
+          if (searchFor.at(0)=='S') searchFor="<u>S</u>"+searchFor.mid(1);
+          t << "    <li>\n";
+          t << "      <form action=\"" << relPath << "search.php\" method=\"get\">\n";
+          t << "        <table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\n";
+          t << "          <tr>\n";
+          t << "            <td><label>&nbsp;" << searchFor << "&nbsp;</label></td>\n";
+          if (!highlightSearch)
+          {
+            t << "            <td><input type=\"text\" name=\"query\" value=\"\" size=\"20\" accesskey=\"s\"/></td>\n";
+            t << "          </tr>\n";
+            t << "        </table>\n";
+            t << "      </form>\n";
+            t << "    </li>\n";
+          }
+        } 
+        if (!highlightSearch) // on the search page the page will be ended by the
+                              // page itself
+        {
+          endQuickIndexList(t,TRUE);
+        }
+      }
+      else // normal case
+      {
+        endQuickIndexList(t,TRUE);
+      }
+    }
+  }
+}
+
+static void writeDefaultQuickLinks(QTextStream &t,bool compact,
+                                   HighlightedItem hli,const QCString &relPath)
+{
+  LayoutNavEntry *root = LayoutDocManager::instance().rootNavEntry();
+  LayoutNavEntry::Kind kind = (LayoutNavEntry::Kind)-1;
+  bool highlightParent=FALSE;
+  switch (hli) // map HLI enums to LayoutNavEntry::Kind enums
+  {
+    case HLI_Main:             kind = LayoutNavEntry::MainPage;         break;
+    case HLI_Modules:          kind = LayoutNavEntry::Modules;          break;
+    case HLI_Directories:      kind = LayoutNavEntry::Dirs;             break;
+    case HLI_Namespaces:       kind = LayoutNavEntry::Namespaces;       break;
+    case HLI_Hierarchy:        kind = LayoutNavEntry::ClassHierarchy;   break;
+    case HLI_Classes:          kind = LayoutNavEntry::Classes;          break;
+    case HLI_Annotated:        kind = LayoutNavEntry::ClassAnnotated;   break;
+    case HLI_Files:            kind = LayoutNavEntry::Files;            break;
+    case HLI_NamespaceMembers: kind = LayoutNavEntry::NamespaceMembers; break;
+    case HLI_Functions:        kind = LayoutNavEntry::ClassMembers;     break;
+    case HLI_Globals:          kind = LayoutNavEntry::FileGlobals;      break;
+    case HLI_Pages:            kind = LayoutNavEntry::Pages;            break;
+    case HLI_Examples:         kind = LayoutNavEntry::Examples;         break;
+    case HLI_ClassVisible:     kind = LayoutNavEntry::Classes;    highlightParent = TRUE; break;
+    case HLI_NamespaceVisible: kind = LayoutNavEntry::Namespaces; highlightParent = TRUE; break;
+    case HLI_FileVisible:      kind = LayoutNavEntry::Files;      highlightParent = TRUE; break;
+    case HLI_None:   break;
+    case HLI_Search: break;
+  }
+  
+  if (compact)
+  {
+    // find highlighted index item
+    LayoutNavEntry *hlEntry = root->find(kind);
+    if (!hlEntry) // highlighted item not found in the index! -> just show the level 1 index...
+    {
+      highlightParent=TRUE;
+      hlEntry = root->children().getFirst();
+      if (hlEntry==0) 
+      {
+        return; // argl, empty index!
+      }
+    }
+    renderQuickLinksAsTabs(t,relPath,hlEntry,kind,highlightParent,hli==HLI_Search);
+  }
+  else
+  {
+    renderQuickLinksAsTree(t,relPath,root);
+  }
+}
+
+#else // old fixed layout
+
 static void writeNamespaceSubIndex(QTextStream &t,bool compact,
                                   HighlightedItem hli,const QCString &relPath
                                  )
 {
+  bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");
   startQuickIndexList(t,compact);
   if (documentedNamespaces>0)
   {
@@ -1632,7 +1924,11 @@ static void writeNamespaceSubIndex(QTextStream &t,bool compact,
     {
       t << fixSpaces(theTranslator->trPackages());
     }
-    else
+    else if(fortranOpt)
+    {
+      t << theTranslator->trModulesList();
+    }
+      else
     {
       t << theTranslator->trNamespaceList();
     }
@@ -1646,7 +1942,11 @@ static void writeNamespaceSubIndex(QTextStream &t,bool compact,
     {
       t << fixSpaces(theTranslator->trPackageMembers());
     }
-    else
+    else if(fortranOpt)
+    {
+      t << fixSpaces(theTranslator->trModulesMembers());
+    }
+      else
     {
       t << fixSpaces(theTranslator->trNamespaceMembers());
     }
@@ -1659,6 +1959,8 @@ static void writeClassSubIndex(QTextStream &t,bool compact,
                                HighlightedItem hli,const QCString &relPath
                               )
 {
+  bool fortranOpt=Config_getBool("OPTIMIZE_FOR_FORTRAN");
+  bool vhdlOpt=Config_getBool("OPTIMIZE_OUTPUT_VHDL");
   startQuickIndexList(t,compact);
   if (annotatedClasses>0)
   {
@@ -1671,21 +1973,46 @@ static void writeClassSubIndex(QTextStream &t,bool compact,
     }
     startQuickIndexItem(t,"annotated"+Doxygen::htmlFileExtension,
         hli==HLI_Annotated,compact,relPath);
-    t << fixSpaces(theTranslator->trCompoundList());
+      if (fortranOpt)
+      {
+        t << fixSpaces(theTranslator->trCompoundListFortran());
+      }
+      else if (vhdlOpt)
+      {
+        t << fixSpaces(VhdlDocGen::trDesignUnitList());
+      }
+      else
+      {
+        t << fixSpaces(theTranslator->trCompoundList());
+      }
+    
     endQuickIndexItem(t);
   } 
   if (hierarchyClasses>0)
   {
+    QCString title = theTranslator->trClassHierarchy();
+    if (vhdlOpt) title = VhdlDocGen::trDesignUnitHierarchy();
     startQuickIndexItem(t,"hierarchy"+Doxygen::htmlFileExtension,
         hli==HLI_Hierarchy,compact,relPath);
-    t << fixSpaces(theTranslator->trClassHierarchy());
+    t << fixSpaces(title);
     endQuickIndexItem(t);
   } 
   if (documentedClassMembers[CMHL_All]>0)
   {
     startQuickIndexItem(t,"functions"+Doxygen::htmlFileExtension,
         hli==HLI_Functions,compact,relPath);
-    t << fixSpaces(theTranslator->trCompoundMembers());
+    if (fortranOpt)
+    {
+      t << fixSpaces(theTranslator->trCompoundMembersFortran());
+    }
+    else if (vhdlOpt)
+    {
+      t << fixSpaces(VhdlDocGen::trDesignUnitMembers());
+    }
+    else
+    {
+      t << fixSpaces(theTranslator->trCompoundMembers());
+    }
     endQuickIndexItem(t);
   } 
   endQuickIndexList(t,compact);
@@ -1715,9 +2042,11 @@ static void writeFileSubIndex(QTextStream &t,bool compact,
 static void writeDefaultQuickLinks(QTextStream &t,bool compact,
                                    HighlightedItem hli,const QCString &relPath)
 {
+  bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");
+  bool vhdlOpt    = Config_getBool("OPTIMIZE_OUTPUT_VHDL");
   startQuickIndexList(t,compact);
 
-  if (Config_getBool("GENERATE_TREEVIEW"))
+  if (usingTreeIndex())
   {
     startQuickIndexItem(t,"main"+Doxygen::htmlFileExtension,
                         hli==HLI_Main,compact,relPath);
@@ -1730,6 +2059,18 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
   t << fixSpaces(theTranslator->trMainPage());
   endQuickIndexItem(t);
 
+  // -------------- Related pages
+  
+  if (indexedPages>0)
+  {
+    startQuickIndexItem(t,"pages"+Doxygen::htmlFileExtension,
+                        hli==HLI_Pages,compact,relPath);
+    t << fixSpaces(theTranslator->trRelatedPages());
+    endQuickIndexItem(t);
+  } 
+
+  // --------------- Modules
+
   if (documentedGroups>0)
   {
     startQuickIndexItem(t,"modules"+Doxygen::htmlFileExtension,
@@ -1749,6 +2090,10 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
     {
       t << fixSpaces(theTranslator->trPackages());
     }
+    else if (fortranOpt)
+    {
+      t << theTranslator->trModules();
+    }
     else
     {
       t << theTranslator->trNamespaces();
@@ -1772,7 +2117,18 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
           hli==HLI_Annotated || hli==HLI_Functions || hli==HLI_ClassVisible,
           compact,relPath);
     }
-    t << fixSpaces(theTranslator->trClasses());
+    if (fortranOpt)
+    {
+      t << fixSpaces(theTranslator->trDataTypes());
+    }
+    else if (vhdlOpt)
+    {
+      t << fixSpaces(VhdlDocGen::trDesignUnits());
+    }
+    else
+    {
+      t << fixSpaces(theTranslator->trClasses());
+    }
     endQuickIndexItem(t);
     if (!compact)
     {
@@ -1780,7 +2136,6 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
     }
   }
 
-
   // -------------- File
 
   if (documentedHtmlFiles>0)
@@ -1806,16 +2161,6 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
     endQuickIndexItem(t);
   } 
 
-  // -------------- Related pages
-  
-  if (indexedPages>0)
-  {
-    startQuickIndexItem(t,"pages"+Doxygen::htmlFileExtension,
-                        hli==HLI_Pages,compact,relPath);
-    t << fixSpaces(theTranslator->trRelatedPages());
-    endQuickIndexItem(t);
-  } 
-
   // -------------- Examples
   
   if (Doxygen::exampleSDict->count()>0)
@@ -1880,6 +2225,33 @@ static void writeDefaultQuickLinks(QTextStream &t,bool compact,
   }
   
 }
+#endif
+
+void HtmlGenerator::startQuickIndices()
+{
+  if (!Config_getBool("DISABLE_INDEX"))
+  {
+    t << "<div class=\"navigation\" id=\"top\">" << endl;
+  }
+}
+
+void HtmlGenerator::endQuickIndices()
+{
+  if (!Config_getBool("DISABLE_INDEX"))
+  {
+    t << "</div>" << endl;
+  }
+}
+
+void HtmlGenerator::startContents()
+{
+  t << "<div class=\"contents\">" << endl;
+}
+
+void HtmlGenerator::endContents()
+{
+  t << "</div>" << endl;
+}
 
 void HtmlGenerator::writeQuickLinks(bool compact,HighlightedItem hli)
 {
@@ -1895,9 +2267,7 @@ void HtmlGenerator::writeSearchPage()
     if (f.open(IO_WriteOnly))
     {
       QTextStream t(&f);
-#if QT_VERSION >= 200
-      t.setEncoding(QTextStream::Latin1);
-#endif
+      t.setEncoding(QTextStream::UnicodeUTF8);
       if (g_header.isEmpty()) 
       {
         writeDefaultHeaderFile(t,theTranslator->trSearch().data(),0,FALSE);
@@ -1966,3 +2336,79 @@ void HtmlGenerator::writeSearchPage()
   }
 }
 
+void HtmlGenerator::generateSectionImages()
+{
+  {
+    QCString fileName = Config_getString("HTML_OUTPUT")+"/open.gif";
+    QFile f(fileName);
+    if (f.open(IO_WriteOnly))
+    {
+      f.writeBlock((char*)open_gif,open_gif_len);
+    }
+  }
+  {
+    QCString fileName = Config_getString("HTML_OUTPUT")+"/closed.gif";
+    QFile f(fileName);
+    if (f.open(IO_WriteOnly))
+    {
+      f.writeBlock((char*)closed_gif,closed_gif_len);
+    }
+  }
+}
+
+void HtmlGenerator::startConstraintList(const char *header)
+{
+  t << "<div class=\"typeconstraint\">" << endl;
+  t << "<dl compact><dt><b>" << header << "</b><dt><dd>" << endl;
+  t << "<table border=\"0\" cellspacing=\"2\" cellpadding=\"0\">" << endl;
+}
+
+void HtmlGenerator::startConstraintParam()
+{
+  t << "<tr><td valign=\"top\"><em>";
+}
+
+void HtmlGenerator::endConstraintParam()
+{
+  t << "</em></td>";
+}
+
+void HtmlGenerator::startConstraintType()
+{
+  t << "<td>&nbsp;:</td><td valign=\"top\"><em>";
+}
+
+void HtmlGenerator::endConstraintType()
+{
+  t << "</em></td>";
+}
+
+void HtmlGenerator::startConstraintDocs()
+{
+  t << "<td>&nbsp;";
+}
+
+void HtmlGenerator::endConstraintDocs()
+{
+  t << "</td></tr>" << endl;
+}
+
+void HtmlGenerator::endConstraintList()
+{
+  t << "</table>" << endl;
+  t << "</dl>" << endl;
+  t << "</div>" << endl;
+}
+
+void HtmlGenerator::lineBreak(const char *style)
+{
+  if (style)
+  {
+    t << "<br class=\"" << style << "\">" << endl;
+  }
+  else
+  {
+    t << "<br>" << endl;
+  }
+}
+
