@@ -54,7 +54,7 @@
 #include "translator_es.h"
 #endif
 #ifdef LANG_FI
-//#include "translator_fi.h"
+#include "translator_fi.h"
 #endif
 #ifdef LANG_RU
 #include "translator_ru.h"
@@ -131,6 +131,9 @@
 #ifdef LANG_MK
 #include "translator_mk.h"
 #endif
+#ifdef LANG_SC
+#include "translator_sc.h"
+#endif
 #endif // !ENGLISH_ONLY
 
 #define L_EQUAL(a) !stricmp(langName,a)
@@ -217,9 +220,7 @@ bool setTranslator(const char *langName)
 #ifdef LANG_FI
   else if (L_EQUAL("finnish"))
   {
-    //theTranslator=new TranslatorFinnish;
-    theTranslator=new TranslatorEnglish;
-    err(obsoleteMsg);
+    theTranslator=new TranslatorFinnish;
   }
 #endif
 #ifdef LANG_RU
@@ -330,10 +331,16 @@ bool setTranslator(const char *langName)
     theTranslator=new TranslatorDecoder(new TranslatorSerbian);
   }
 #endif
+#ifdef LANG_SC
+  else if (L_EQUAL("serbian-cyrilic"))
+  {
+    theTranslator=new TranslatorSerbian;
+  }
+#endif
 #ifdef LANG_CA
   else if (L_EQUAL("catalan"))
   {
-    theTranslator=new TranslatorDecoder(new TranslatorCatalan);
+    theTranslator=new TranslatorCatalan;
   }
 #endif
 #ifdef LANG_LT
