@@ -236,7 +236,7 @@ static void writeDefaultHeaderPart1(QTextStream &t)
     s=extraPackages.next();
   }
   t << "\\makeindex\n"
-    "\\setcounter{tocdepth}{1}\n"
+    "\\setcounter{tocdepth}{3}\n"
     "\\renewcommand{\\footrulewidth}{0.4pt}\n"
     "\\begin{document}\n";
   if (theTranslator->idLanguage()=="greek") t << "\\selectlanguage{greek}\n";
@@ -585,6 +585,8 @@ void LatexGenerator::startIndexSection(IndexSections is)
         t << "{"; //Page Documentation}\n";
       }
       break;
+    case isPageDocumentation2:
+      break;
     case isEndIndex:
       break;
   }
@@ -606,8 +608,7 @@ void LatexGenerator::endIndexSection(IndexSections is)
       break;
     case isMainPage:
       {
-        QCString indexName="index";
-        if (Config_getBool("GENERATE_TREEVIEW")) indexName="main";
+        QCString indexName=usingTreeIndex()?"main":"index";
         t << "}\n\\label{index}";
         if (Config_getBool("PDF_HYPERLINKS")) t << "\\hypertarget{index}{}";
         t << "\\input{" << indexName << "}\n";
@@ -794,6 +795,8 @@ void LatexGenerator::endIndexSection(IndexSections is)
 #endif
       }
       break;
+    case isPageDocumentation2:
+      break;
     case isEndIndex:
       t << "\\printindex\n";
       t << "\\end{document}\n";
@@ -1105,7 +1108,7 @@ void LatexGenerator::endMemberHeader()
 void LatexGenerator::startMemberDoc(const char *clname,
                                     const char *memname,
                                     const char *,
-                                    const char *)
+                                    const char *title)
 { 
   if (memname && memname[0]!='@')
   {
@@ -1142,6 +1145,9 @@ void LatexGenerator::startMemberDoc(const char *clname,
   //  escapeMakeIndexChars(this,t,memname);
   //  t << "]";
   //}
+  t << "[{";
+  escapeMakeIndexChars(title);
+  t << "}]";
   t << "{\\setlength{\\rightskip}{0pt plus 5cm}";
   disableLinks=TRUE;
 }
@@ -1182,9 +1188,16 @@ void LatexGenerator::writeAnchor(const char *fName,const char *name)
 { 
   //printf("LatexGenerator::writeAnchor(%s,%s)\n",fName,name);
   t << "\\label{" << name << "}" << endl; 
-  if (fName && Config_getBool("PDF_HYPERLINKS"))
+  if (Config_getBool("PDF_HYPERLINKS"))
   {
-    t << "\\hypertarget{" << stripPath(fName) << "_" << name << "}{}" << endl;
+    if (fName)
+    {
+      t << "\\hypertarget{" << stripPath(fName) << "_" << name << "}{}" << endl;
+    }
+    else
+    {
+      t << "\\hypertarget{" << name << "}{}" << endl;
+    }
   }
 }
 
@@ -1648,8 +1661,8 @@ void LatexGenerator::escapeLabelName(const char *s)
     switch (c)
     {
       case '%': t << "\\%";       break;
-      case '|': t << "\\tt{\"|}"; break;
-      case '!': t << "\"!";       break;
+      //case '|': t << "\\tt{\"|}"; break;
+      //case '!': t << "\"!";       break;
       default:  str[0]=c; docify(str); break;
     }
   }
@@ -1665,10 +1678,10 @@ void LatexGenerator::escapeMakeIndexChars(const char *s)
   {
     switch (c)
     {
-      case '!': t << "\"!"; break;
+      //case '!': t << "\"!"; break;
       case '"': t << "\"\""; break;
       case '@': t << "\"@"; break;
-      case '|': t << "\\tt{\"|}"; break;
+      //case '|': t << "\\tt{\"|}"; break;
       case '[': t << "["; break;
       case ']': t << "]"; break;
       default:  str[0]=c; docify(str); break;
