@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -807,6 +807,11 @@ void ManDocVisitor::visitPre(DocParamSect *s)
       m_t << theTranslator->trReturnValues(); break;
     case DocParamSect::Exception: 
       m_t << theTranslator->trExceptions(); break;
+    case DocParamSect::TemplateParam: 
+      /* TODO: add this 
+      m_t << theTranslator->trTemplateParam(); break;
+      */
+      m_t << "Template Parameters"; break;
     default:
       ASSERT(0);
   }
