@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.cpp,v 1.17 1998/11/28 11:33:19 root Exp $
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -179,6 +179,21 @@ void ManGenerator::newParagraph()
   paragraph=TRUE;
 }
 
+void ManGenerator::startParagraph()
+{
+  if (!paragraph)
+  {
+    if (!firstCol) t << endl;
+    t << ".PP" << endl;
+    firstCol=TRUE;
+  }
+  paragraph=TRUE;
+}
+
+void ManGenerator::endParagraph()
+{
+}
+
 void ManGenerator::writeString(const char *text)
 {
   docify(text);
@@ -401,9 +416,7 @@ void ManGenerator::startDoxyAnchor(const char *,const char *manName,
         {
 	      QTextStream linkstream;
 	      linkstream.setDevice(&linkfile);
-#if QT_VERSION >= 200
-	      linkstream.setEncoding(QTextStream::Latin1);
-#endif
+	      linkstream.setEncoding(QTextStream::UnicodeUTF8);
 	      linkstream << ".so man" << getExtension() << "/" << buildFileName( manName ) << endl;
 	}
     }
@@ -640,3 +653,54 @@ void ManGenerator::printDoc(DocNode *n,const char *langExt)
   paragraph = FALSE;
 }
 
+void ManGenerator::startConstraintList(const char *header)
+{
+  if (!firstCol) 
+  { t << endl << ".PP" << endl; 
+    firstCol=TRUE; paragraph=TRUE; 
+    col=0;
+  }
+  paragraph=FALSE;
+  startBold();
+  docify(header);
+  endBold();
+  paragraph=TRUE;
+}
+
+void ManGenerator::startConstraintParam()
+{
+  writeListItem();
+  startEmphasis();
+}
+
+void ManGenerator::endConstraintParam()
+{
+  endEmphasis();
+  t << " : ";
+}
+
+void ManGenerator::startConstraintType()
+{
+  startEmphasis();
+}
+
+void ManGenerator::endConstraintType()
+{
+  endEmphasis();
+}
+
+void ManGenerator::startConstraintDocs()
+{
+}
+
+void ManGenerator::endConstraintDocs()
+{
+  t << endl; firstCol=TRUE;
+}
+
+void ManGenerator::endConstraintList()
+{
+}
+
+
+
