@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -57,7 +57,6 @@ class MemberDef : public Definition
       Typedef, 
       Enumeration, 
       EnumValue,
-      Prototype,
       Signal,
       Slot,
       Friend,
@@ -69,7 +68,7 @@ class MemberDef : public Definition
     MemberDef(const char *defFileName,int defLine,
               const char *type,const char *name,const char *args,
               const char *excp,Protection prot,Specifier virt,bool stat,
-              bool related,MemberType t,const ArgumentList *tal,
+              Relationship related,MemberType t,const ArgumentList *tal,
               const ArgumentList *al);
    ~MemberDef(); 
     DefType definitionType() const        { return TypeMember; }
@@ -89,6 +88,7 @@ class MemberDef : public Definition
     const char *argsString() const;
     const char *excpString() const;
     const char *bitfieldString() const;
+    const char *extraTypeChars() const;
     const QCString &initializer() const;
     int initializerLines() const;
     int  getMemberSpecifiers() const;
@@ -109,7 +109,7 @@ class MemberDef : public Definition
     const char *getGroupFileName() const;
     int getGroupStartLine() const;
     bool getGroupHasDocs() const;
-    QCString qualifiedName();
+    QCString qualifiedName() const;
     QCString objCMethodName(bool localLink,bool showStatic) const; 
 
     // direct kind info 
@@ -132,6 +132,7 @@ class MemberDef : public Definition
     bool isProperty() const;
     bool isEvent() const;
     bool isRelated() const;
+    bool isForeign() const;
     bool isStatic() const;
     bool isInline() const;
     bool isExplicit() const;
@@ -147,6 +148,12 @@ class MemberDef : public Definition
     bool isAbstract() const;
     bool isOverride() const;
     bool isInitonly() const;
+    bool isOptional() const;
+    bool isRequired() const;
+    bool isNonAtomic() const;
+    bool isCopy() const;
+    bool isAssign() const;
+    bool isRetain() const;
     bool isNew() const;
     bool isSealed() const;
     bool isImplementation() const;
@@ -165,6 +172,7 @@ class MemberDef : public Definition
     bool isLinkableInProject() const;
     bool isLinkable() const;
     bool hasDocumentation() const;  // overrides hasDocumentation in definition.h
+    bool hasUserDocumentation() const; // overrides hasUserDocumentation
     bool isBriefSectionVisible() const;
     bool isDetailedSectionVisible(bool inGroup,bool inFile) const;
     bool isDetailedSectionLinkable() const;
@@ -249,6 +257,7 @@ class MemberDef : public Definition
     void setTemplateSpecialization(bool b);
     
     void makeRelated();
+    void makeForeign();
     void setHasDocumentedParams(bool b);
     void setHasDocumentedReturnType(bool b);
     void setInheritsDocsFrom(MemberDef *md);
@@ -281,6 +290,7 @@ class MemberDef : public Definition
     void setArgumentList(ArgumentList *al);
     void setDeclArgumentList(ArgumentList *al);
     void setDefinitionTemplateParameterLists(QList<ArgumentList> *lists);
+    void setTypeConstraints(ArgumentList *al);
 
     // namespace related members
     void setNamespace(NamespaceDef *nd);
