@@ -26,6 +26,7 @@
 #include "outputlist.h"
 #include "groupdef.h"
 #include "marshal.h"
+#include "vhdldocgen.h"
 
 MemberList::MemberList()
 {
@@ -87,7 +88,7 @@ void MemberList::countDecMembers(bool countEnumValues)
                                        m_enumValCnt++,m_numDecMembers++; 
                                      break;
         case MemberDef::Typedef:     m_typeCnt++,m_numDecMembers++; break;
-        case MemberDef::Prototype:   m_protoCnt++,m_numDecMembers++; break;
+        //case MemberDef::Prototype:   m_protoCnt++,m_numDecMembers++; break;
         case MemberDef::Define:      if (Config_getBool("EXTRACT_ALL") || 
                                          md->argsString() || 
                                          !md->initializer().isEmpty() ||
@@ -200,7 +201,7 @@ void MemberList::writePlainDeclarations(OutputList &ol,
       switch(md->memberType())
       {
         case MemberDef::Define:    // fall through
-        case MemberDef::Prototype: // fall through
+        //case MemberDef::Prototype: // fall through
         case MemberDef::Typedef:   // fall through
         case MemberDef::Variable:  // fall through
         case MemberDef::Function:  // fall through
@@ -319,7 +320,14 @@ void MemberList::writeDeclarations(OutputList &ol,
              /*, bool inGroup,bool countSubGroups*/)
 {
   //printf("----- writeDeclaration() this=%p ----\n",this);
+  static bool optimizeVhdl = Config_getBool("OPTIMIZE_OUTPUT_VHDL");
+
   countDecMembers(showEnumValues); // count member not in group
+  Definition *ctx = cd;
+  if (ctx==0 && nd) ctx = nd;
+  if (ctx==0 && gd) ctx = gd;
+  if (ctx==0 && fd) ctx = fd;
+
   if (numDecMembers()==0) return;
   //printf("%p: MemberList::writeDeclaration(title=`%s',subtitle=`%s')=%d\n",
   //    this,title,subtitle,numDecMembers());
@@ -333,11 +341,25 @@ void MemberList::writeDeclarations(OutputList &ol,
   {
     //printf("subtitle=`%s'\n",subtitle);
     ol.startMemberSubtitle();
-    ol.parseDoc("[generated]",-1,0,0,subtitle,FALSE,FALSE);
+    ol.parseDoc("[generated]",-1,ctx,0,subtitle,FALSE,FALSE);
     ol.endMemberSubtitle();
   }
 
-  writePlainDeclarations(ol,cd,nd,fd,gd);
+  // TODO: Two things need to be worked out for proper VHDL output:
+  // 1. Signals and types under the group need to be
+  //    formatted to associate them with the group somehow
+  //    indentation, or at the very least, extra space after
+  //    the group is done
+  // 2. This might need to be repeated below for memberGroupLists
+  if (optimizeVhdl) // use specific declarations function
+  {
+    VhdlDocGen::writeVhdlDeclarations(this,ol,0,cd,0);
+  }
+  else
+  {
+    writePlainDeclarations(ol,cd,nd,fd,gd);
+  }
+ 
   
   //printf("memberGroupList=%p\n",memberGroupList);
   if (memberGroupList)
@@ -358,7 +380,7 @@ void MemberList::writeDeclarations(OutputList &ol,
       {
         //printf("Member group has docs!\n");
         ol.startMemberGroupDocs();
-        ol.parseDoc("[generated]",-1,0,0,mg->documentation()+"\n",FALSE,FALSE);
+        ol.parseDoc("[generated]",-1,ctx,0,mg->documentation()+"\n",FALSE,FALSE);
         ol.endMemberGroupDocs();
       }
       ol.startMemberGroup();
