@@ -2,7 +2,7 @@
  *
  * $Id: msc.h,v 1.14 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -26,7 +26,7 @@ void writeMscGraphFromFile(const char *inFile,const char *outDir,
                            const char *outFile,MscOutputFormat format);
 
 QString getMscImageMapFromFile(const QString& inFile, const QString& outDir,
-                               const QCString& relPath);
+                               const QCString& relPath,const QString& context);
 
 #endif
 
