@@ -546,7 +546,7 @@ void PerlModDocVisitor::visit(DocSymbol *sy)
   case DocSymbol::Rdquo:   c = '"'; break;
   case DocSymbol::Ndash:   c = '-'; break;
   case DocSymbol::Mdash:   s = "--"; break;
-  case DocSymbol::Nbsp:    c = ''; break;
+  case DocSymbol::Nbsp:    c = ' '; break;
   case DocSymbol::Uml:     accent = "umlaut"; break;
   case DocSymbol::Acute:   accent = "acute"; break;
   case DocSymbol::Grave:   accent = "grave"; break;
@@ -1430,7 +1430,7 @@ void PerlModGenerator::generatePerlModForMember(MemberDef *md,Definition *)
   case MemberDef::Enumeration: memType="enum";      break;
   case MemberDef::Function:    memType="function";  isFunc=TRUE; break;
   case MemberDef::Signal:      memType="signal";    isFunc=TRUE; break;
-  case MemberDef::Prototype:   memType="prototype"; isFunc=TRUE; break;
+  //case MemberDef::Prototype:   memType="prototype"; isFunc=TRUE; break;
   case MemberDef::Friend:      memType="friend";    isFunc=TRUE; break;
   case MemberDef::DCOP:        memType="dcop";      isFunc=TRUE; break;
   case MemberDef::Slot:        memType="slot";      isFunc=TRUE; break;
