@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2004 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -14,6 +14,8 @@
  * input used in their production; they are not affected by this license.
  *
  * The translation into German was provided by
+ *   Jens Seidel (jensseidel@users.sf.net)
+ * based on work from
  *   Jens Breitenstein (Jens.Breitenstein@tlc.de)
  */
 
@@ -96,6 +98,22 @@
 //    - added a dash after (file, ...) names and in front of description
 //    - changed "Eigenschaften" to "Propertys" (yeah, not ..."ies")
 //    
+//   2005/03/20 Jens Seidel (jensseidel@users.sourceforge.net)
+//    - Updated for "new since 1.4.1" version
+//    
+//   2005/04/09 Jens Seidel (jensseidel@users.sourceforge.net)
+//    - Changed Todo list translation as suggested in
+//      http://bugzilla.gnome.org/show_bug.cgi?id=172818
+//    
+//   2005/05/09 Jens Seidel (jensseidel@users.sourceforge.net)
+//    - Updated for "new since 1.4.3" version (removed unused methods)
+//
+//   2006/06/12 Jens Seidel (jensseidel@users.sourceforge.net)
+//    - Updated for "new since 1.4.6" version
+//
+//   2008/02/04 Jens Seidel (jensseidel@users.sourceforge.net)
+//    - Updated for "new since 1.5.4" version
+//
 //   Todo:
 //    - see FIXME
 
@@ -122,9 +140,10 @@ class TranslatorGerman : public Translator
      */
     virtual QCString latexLanguageSupportCommand()
     {
-      QCString result="\\usepackage{ngerman}\n";
-      result+="\\usepackage[latin1]{inputenc}\n";
+      //QCString result="\\usepackage{ngerman}\n";
+      //result+="\\usepackage[latin1]{inputenc}\n";
       //result+="%\\usepackage[latin1]{inputenc}% Kodierung (cp850,latin1,ansinew)\n";
+      QCString result="\\usepackage[german]{babel}\n";
       return result;
     }
 
@@ -237,10 +256,6 @@ class TranslatorGerman : public Translator
     virtual QCString trFileList()
     { return "Auflistung der Dateien"; }
 
-    /*! This is put above each page as a link to the list of all verbatim headers */
-    virtual QCString trHeaderFiles()
-    { return "Header-Dateien"; }
-
     /*! This is put above each page as a link to all members of compounds. */
     virtual QCString trCompoundMembers()
     { 
@@ -364,10 +379,6 @@ class TranslatorGerman : public Translator
       return result;
     }
 
-    /*! This is an introduction to the page with the list of all header files. */
-    virtual QCString trHeaderFilesDescription()
-    { return "Hier folgen die Headerdateien, welche die API definieren:"; }
-
     /*! This is an introduction to the page with the list of all examples */
     virtual QCString trExamplesDescription()
     { return "Hier folgt eine Liste mit allen Beispielen:"; }
@@ -380,12 +391,6 @@ class TranslatorGerman : public Translator
     virtual QCString trModulesDescription()
     { return "Hier folgt die Aufzhlung aller Module:"; }
 
-    /*! This sentences is used in the annotated class/file lists if no brief
-     * description is given. 
-     */
-    virtual QCString trNoDescriptionAvailable()
-    { return "Keine Beschreibung verfgbar"; }
-
     // index titles (the project name is prepended for these) 
 
     /*! This is used in HTML as the title of index.html. */
@@ -530,12 +535,6 @@ class TranslatorGerman : public Translator
     virtual QCString trEnumerationTypeDocumentation()
     { return "Dokumentation der Aufzhlungstypen"; }
 
-    /*! This is used in the documentation of a file/namespace before the list 
-     *  of documentation blocks for enumeration values
-     */
-    virtual QCString trEnumerationValueDocumentation()
-    { return "Dokumentation des Wertebereiches der Aufzhlungstypen"; }
-
     /*! This is used in the documentation of a file/namespace before the list 
      *  of documentation blocks for functions
      */
@@ -591,19 +590,10 @@ class TranslatorGerman : public Translator
     virtual QCString trForInternalUseOnly()
     { return "Nur fr den internen Gebrauch."; }
 
-    /*! this text is generated when the \\reimp command is used. */
-    virtual QCString trReimplementedForInternalReasons()
-    { return "Aus internen Grnden neu implementiert. "
-             "Das API wird davon nicht berhrt."; }
-
     /*! this text is generated when the \\warning command is used. */
     virtual QCString trWarning()
     { return "Warnung"; }
 
-    /*! this text is generated when the \\bug command is used. */
-    virtual QCString trBugsAndLimitations()
-    { return "Fehler und Einschrnkungen"; }
-
     /*! this text is generated when the \\version command is used. */
     virtual QCString trVersion()
     { return "Version"; }
@@ -682,8 +672,8 @@ class TranslatorGerman : public Translator
         case ClassDef::Struct: result+="Struktur"; break;
         case ClassDef::Union:  result+="Varianten"; break;
         case ClassDef::Interface:  result+="Schnittstellen"; break;
-        case ClassDef::Protocol:  result+="Protocol"; break; // translate me!
-        case ClassDef::Category:  result+="Category"; break; // translate me!
+        case ClassDef::Protocol:  result+="Protokoll"; break;
+        case ClassDef::Category:  result+="Kategorie"; break; 
         case ClassDef::Exception:  result+="Ausnahmen"; break;
       }
       result+="referenz";
@@ -850,8 +840,8 @@ class TranslatorGerman : public Translator
         case ClassDef::Struct:     result+=" Struktur"; break;
         case ClassDef::Union:      result+=" Variante"; break;
         case ClassDef::Interface:  result+=" Schnittstelle"; break;
-        case ClassDef::Protocol:   result+=" Protocol"; break; // translate me!
-        case ClassDef::Category:   result+=" Category"; break; // translate me!
+        case ClassDef::Protocol:   result+="s Protokoll"; break; 
+        case ClassDef::Category:   result+=" Kategorie"; break;
         case ClassDef::Exception:  result+=" Ausnahme"; break;
       }
       result+=" wurde erzeugt aufgrund der Datei";
@@ -888,11 +878,6 @@ class TranslatorGerman : public Translator
 // new since 0.49-991003
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trSources()
-    {
-      return "Quellen";
-    }
-
     virtual QCString trDefinedAtLineInSourceFile()
     {
       return "Definiert in Zeile @0 der Datei @1.";
@@ -1071,7 +1056,7 @@ class TranslatorGerman : public Translator
     /*! Used as the header of the todo list */
     virtual QCString trTodoList()
     {
-      return "Liste der zu erledigenden Dinge";
+      return "Ausstehende Aufgaben";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1144,7 +1129,7 @@ class TranslatorGerman : public Translator
         "class Inherited : public PublicBase,\n"
         "                  protected ProtectedBase,\n"
         "                  private PrivateBase,\n"
-        "                  public Undocumented\n"
+        "                  public Undocumented,\n"
         "                  public Templ<int>\n"
         "{\n"
         "  private:\n"
@@ -1236,12 +1221,6 @@ class TranslatorGerman : public Translator
 // new since 1.2.4
 //////////////////////////////////////////////////////////////////////////
 
-    /*! Used for Java interfaces in the summary section of Java packages */
-    virtual QCString trInterfaces()
-    {
-      return "Schnittstellen";
-    }
-
     /*! Used for Java classes in the summary section of Java packages */
     virtual QCString trClasses()
     {
@@ -1275,12 +1254,6 @@ class TranslatorGerman : public Translator
       return "Pakete";
     }
 
-    /*! Used as a chapter title for Latex & RTF output */
-    virtual QCString trPackageDocumentation()
-    {
-      return "Paketdokumentation";
-    }
-
     /*! Text shown before a multi-line define */
     virtual QCString trDefineValue()
     {
@@ -1397,17 +1370,6 @@ class TranslatorGerman : public Translator
       return result;
     }
    
-    /*! This is used for translation of the word that will possibly
-     *  be followed by a single name or by a list of names 
-     *  of the category.
-     */
-    virtual QCString trField(bool, bool singular)
-    {
-      QCString result("Feld");
-      if (!singular)  result+="er";
-      return result;
-    }
-
     /*! This is used for translation of the word that will possibly
      *  be followed by a single name or by a list of names 
      *  of the category.
@@ -1563,7 +1525,7 @@ class TranslatorGerman : public Translator
     /*! Put in front of the call graph for a function. */
     virtual QCString trCallGraph()
     {
-      return "Hier ist der Graph aller Aufrufe fr diese Funktion:";
+      return "Hier ist ein Graph der zeigt, was diese Funktion aufruft:";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1645,7 +1607,7 @@ class TranslatorGerman : public Translator
     { return "Verzeichnisdokumentation"; }
 
     /*! This is used as the title of the directory index and also in the
-     *  Quick links of a HTML page, to link to the directory hierarchy.
+     *  Quick links of an HTML page, to link to the directory hierarchy.
      */
     virtual QCString trDirectories()
     { return "Verzeichnisse"; }
@@ -1674,6 +1636,237 @@ class TranslatorGerman : public Translator
       return result; 
     }
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.1
+//////////////////////////////////////////////////////////////////////////
+
+    /*! This text is added to the documentation when the \\overload command
+     *  is used for a overloaded function.
+     */
+    virtual QCString trOverloadText()
+    {
+       return "Dies ist eine berladene Methode, die der Bequemlichkeit "
+              "wegen bereitgestellt wird. Sie unterscheidet sich von der "
+              "obigen Funktion nur in den Argumenten die sie untersttzt.";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.6
+//////////////////////////////////////////////////////////////////////////
+
+    /*! This is used to introduce a caller (or called-by) graph */
+    virtual QCString trCallerGraph()
+    {
+      return "Hier ist ein Graph der zeigt, wo diese Funktion aufgerufen wird:";
+    }
+
+    /*! This is used in the documentation of a file/namespace before the list 
+     *  of documentation blocks for enumeration values
+     */
+    virtual QCString trEnumerationValueDocumentation()
+    { return "Enumerator-Dokumentation"; }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.5.4 (mainly for Fortran)
+//////////////////////////////////////////////////////////////////////////
+    
+    /*! header that is put before the list of member subprograms (Fortran). */
+    virtual QCString trMemberFunctionDocumentationFortran()
+    { return "Elementfunktionen/Unterroutinen-Dokumentation"; }
+
+    /*! This is put above each page as a link to the list of annotated data types (Fortran). */    
+    virtual QCString trCompoundListFortran()
+    { return "Datentyp-Liste"; }
+
+    /*! This is put above each page as a link to all members of compounds (Fortran). */
+    virtual QCString trCompoundMembersFortran()
+    { return "Datenfelder"; }
+
+    /*! This is an introduction to the annotated compound list (Fortran). */
+    virtual QCString trCompoundListDescriptionFortran()
+	  { return "Hier folgen die Datentypen mit Kurzbeschreibungen:"; }
+
+    /*! This is an introduction to the page with all data types (Fortran). */
+    virtual QCString trCompoundMembersDescriptionFortran(bool extractAll)
+    {
+      QCString result="Hier folgt eine Liste aller ";
+      if (!extractAll)
+      {
+        result+="dokumentierten ";
+      }
+      result+="Datentypelemente";
+      result+=" mit Links ";
+      if (!extractAll) 
+      {
+         result+="zur Datenstruktur-Dokumentation fr jedes Element";
+      }
+      else 
+      {
+         result+="zu den Datentypen, zu denen sie gehren:";
+      }
+      return result;
+    }
+
+    /*! This is used in LaTeX as the title of the chapter with the 
+     * annotated compound index (Fortran).
+     */
+    virtual QCString trCompoundIndexFortran()
+    { return "Datentyp-Index"; }
+
+    /*! This is used in LaTeX as the title of the chapter containing
+     *  the documentation of all data types (Fortran).
+     */
+    virtual QCString trTypeDocumentation()
+    { return "Datentyp-Dokumentation"; }
+
+    /*! This is used in the documentation of a file as a header before the 
+     *  list of (global) subprograms (Fortran).
+     */
+    virtual QCString trSubprograms()
+    { return "Funktionen/Unterroutinen"; }
+
+    /*! This is used in the documentation of a file/namespace before the list 
+     *  of documentation blocks for subprograms (Fortran)
+     */
+    virtual QCString trSubprogramDocumentation()
+    { return "Funktionen/Unterroutinen-Dokumentation"; }
+
+    /*! This is used in the documentation of a file/namespace/group before 
+     *  the list of links to documented compounds (Fortran)
+     */
+     virtual QCString trDataTypes()
+    { return "Datentypen"; }
+    
+    /*! used as the title of page containing all the index of all modules (Fortran). */
+    virtual QCString trModulesList()
+    { return "Modulliste"; }
+
+    /*! used as an introduction to the modules list (Fortran) */
+    virtual QCString trModulesListDescription(bool extractAll)
+    {
+      QCString result="Hier folgt eine Liste aller ";
+      if (!extractAll) result+="dokumentierten ";
+      result+="Module mit ihren Kurzbeschreibungen:";
+      return result;
+    }
+
+    /*! used as the title of the HTML page of a module/type (Fortran) */
+    virtual QCString trCompoundReferenceFortran(const char *clName,
+                                    ClassDef::CompoundType compType,
+                                    bool isTemplate)
+    {
+      QCString result=(QCString)clName;
+      result += "-";
+      switch(compType)
+      {
+        case ClassDef::Class:      result+="Modul"; break;
+        case ClassDef::Struct:     result+="Typ"; break;
+        case ClassDef::Union:      result+="Union"; break;
+        case ClassDef::Interface:  result+="Interface"; break;
+        case ClassDef::Protocol:   result+="Protokoll"; break;
+        case ClassDef::Category:   result+="Kategorie"; break;
+        case ClassDef::Exception:  result+="Ausnahmen"; break;
+      }
+      if (isTemplate) result+="-Template";
+      result+="-Referenz";
+      return result;
+    }
+    /*! used as the title of the HTML page of a module (Fortran) */
+    virtual QCString trModuleReference(const char *namespaceName)
+    {
+      QCString result=namespaceName;
+      result+="-Modul-Referenz";
+      return result;
+    }
+    
+    /*! This is put above each page as a link to all members of modules. (Fortran) */
+    virtual QCString trModulesMembers()
+    { return "Modul-Elemente"; }
+
+    /*! This is an introduction to the page with all modules members (Fortran) */
+    virtual QCString trModulesMemberDescription(bool extractAll)
+    { 
+      QCString result="Hier folgt eine Liste aller ";
+      if (!extractAll) result+="dokumentierten ";
+      result+="Modulelemente mit Links ";
+      if (extractAll) 
+      {
+        result+="zur Moduldokumentation fr jedes Element:";
+      }
+      else 
+      {
+        result+="zu den Modulen, zu denen sie gehren:";
+      }
+      return result;
+    }
+
+    /*! This is used in LaTeX as the title of the chapter with the 
+     *  index of all modules (Fortran).
+     */
+    virtual QCString trModulesIndex()
+    { return "Modul-Index"; }
+    
+    /*! This is used for translation of the word that will possibly
+     *  be followed by a single name or by a list of names 
+     *  of the category.
+     */
+    virtual QCString trModule(bool /*first_capital*/, bool singular)
+    {       
+      QCString result("Modul");
+      if (!singular)  result+="e";
+      return result; 
+    }
+    /*! This is put at the bottom of a module documentation page and is
+     *  followed by a list of files that were used to generate the page.
+     */
+    virtual QCString trGeneratedFromFilesFortran(ClassDef::CompoundType compType,
+        bool single)
+    {
+      // single is true implies a single file
+      QCString result=(QCString)"Die Dokumentation fr ";
+      switch(compType)
+      {
+        case ClassDef::Class:      result+="dieses Modul"; break;
+        case ClassDef::Struct:     result+="diesen Typ"; break;
+        case ClassDef::Union:      result+="diese Union"; break;
+        case ClassDef::Interface:  result+="dieses Interface"; break;
+        case ClassDef::Protocol:   result+="dieses Protokoll"; break;
+        case ClassDef::Category:   result+="diese Kategorie"; break;
+        case ClassDef::Exception:  result+="diese Ausnahme"; break;
+      }
+      if (single)
+        result+=" wurde aus der folgenden Datei erzeugt:";
+      else
+        result+=" wurde aus den folgenden Dateien erzeugt:";
+      return result;
+    }
+    /*! This is used for translation of the word that will possibly
+     *  be followed by a single name or by a list of names 
+     *  of the category.
+     */
+    virtual QCString trType(bool /*first_capital*/, bool singular)
+    { 
+      QCString result("Typ");
+      if (!singular)  result+="en";
+      return result; 
+    }
+    /*! This is used for translation of the word that will possibly
+     *  be followed by a single name or by a list of names 
+     *  of the category.
+     */
+    virtual QCString trSubprogram(bool /*first_capital*/, bool singular)
+    { 
+      QCString result("Unterprogramm");
+      if (!singular)  result+="e";
+      return result; 
+    }
+
+    /*! C# Type Constraint list */
+    virtual QCString trTypeConstraints()
+    {
+      return "Type Constraints";
+    }
+
 };
 
 #endif
