@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -100,6 +100,7 @@ class TranslatorDecoder : public Translator
     { return toUtf8(m_translator->trMemberTypedefDocumentation()); }
     QCString trMemberEnumerationDocumentation()  
     { return toUtf8(m_translator->trMemberEnumerationDocumentation()); }
+        
     QCString trMemberFunctionDocumentation()  
     { return toUtf8(m_translator->trMemberFunctionDocumentation()); }
     QCString trMemberDataDocumentation()  
@@ -131,6 +132,7 @@ class TranslatorDecoder : public Translator
     { return toUtf8(m_translator->trClassHierarchy()); }
     QCString trCompoundList()  
     { return toUtf8(m_translator->trCompoundList()); }
+    
     QCString trFileList()  
     { return toUtf8(m_translator->trFileList()); }
     QCString trCompoundMembers()  
@@ -213,11 +215,11 @@ class TranslatorDecoder : public Translator
     QCString trCompounds()  
     { return toUtf8(m_translator->trCompounds()); }
     QCString trGeneratedAt(const char *date,const char *projName)  
-    { return toUtf8(m_translator->trGeneratedAt(date,fromUtf8(projName))); }
+    { return toUtf8(m_translator->trGeneratedAt(fromUtf8(date),fromUtf8(projName))); }
     QCString trWrittenBy()  
     { return toUtf8(m_translator->trWrittenBy()); }
     QCString trClassDiagram(const char *clName)  
-    { return toUtf8(m_translator->trClassDiagram(clName)); }
+    { return toUtf8(m_translator->trClassDiagram(fromUtf8(clName))); }
     QCString trForInternalUseOnly()  
     { return toUtf8(m_translator->trForInternalUseOnly()); }
     QCString trWarning()  
@@ -262,12 +264,13 @@ class TranslatorDecoder : public Translator
     QCString trCompoundReference(const char *clName,
                                  ClassDef::CompoundType compType,
                                  bool isTemplate)  
-    { return toUtf8(m_translator->trCompoundReference(clName,compType,isTemplate)); }
+    { return toUtf8(m_translator->trCompoundReference(fromUtf8(clName),compType,isTemplate)); }
 
+    
     QCString trFileReference(const char *fileName)  
     { return toUtf8(m_translator->trFileReference(fromUtf8(fileName))); }
     QCString trNamespaceReference(const char *namespaceName)  
-    { return toUtf8(m_translator->trNamespaceReference(namespaceName)); }
+    { return toUtf8(m_translator->trNamespaceReference(fromUtf8(namespaceName))); }
     
     QCString trPublicMembers()  
     { return toUtf8(m_translator->trPublicMembers()); }
@@ -303,6 +306,7 @@ class TranslatorDecoder : public Translator
     { return toUtf8(m_translator->trNamespaceMembers()); }
     QCString trNamespaceMemberDescription(bool extractAll)  
     { return toUtf8(m_translator->trNamespaceMemberDescription(extractAll)); }
+    
     QCString trNamespaceIndex()  
     { return toUtf8(m_translator->trNamespaceIndex()); }
     QCString trNamespaceDocumentation()  
@@ -322,6 +326,8 @@ class TranslatorDecoder : public Translator
     QCString trGeneratedFromFiles(ClassDef::CompoundType compType,
                                           bool single)  
     { return toUtf8(m_translator->trGeneratedFromFiles(compType,single)); }
+    
+    
     QCString trAlphabeticalList()  
     { return toUtf8(m_translator->trAlphabeticalList()); }
 
@@ -357,7 +363,7 @@ class TranslatorDecoder : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     QCString trCollaborationDiagram(const char *clName)  
-    { return toUtf8(m_translator->trCollaborationDiagram(clName)); }
+    { return toUtf8(m_translator->trCollaborationDiagram(fromUtf8(clName))); }
     QCString trInclDepGraph(const char *fName)  
     { return toUtf8(m_translator->trInclDepGraph(fromUtf8(fName))); }
     QCString trConstructorDocumentation()  
@@ -478,7 +484,7 @@ class TranslatorDecoder : public Translator
     QCString trClasses()  
     { return toUtf8(m_translator->trClasses()); }
     QCString trPackage(const char *name)  
-    { return toUtf8(m_translator->trPackage(name)); }
+    { return toUtf8(m_translator->trPackage(fromUtf8(name))); }
     QCString trPackageList()  
     { return toUtf8(m_translator->trPackageList()); }
     QCString trPackageListDescription()  
@@ -516,6 +522,7 @@ class TranslatorDecoder : public Translator
     { return toUtf8(m_translator->trFile(first_capital,singular)); }
     QCString trNamespace(bool first_capital, bool singular)  
     { return toUtf8(m_translator->trNamespace(first_capital,singular)); }
+    
     QCString trGroup(bool first_capital, bool singular)  
     { return toUtf8(m_translator->trGroup(first_capital,singular)); }
     QCString trPage(bool first_capital, bool singular)  
@@ -648,6 +655,60 @@ class TranslatorDecoder : public Translator
     QCString trEnumerationValueDocumentation()  
     { return toUtf8(m_translator->trEnumerationValueDocumentation()); }
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.5.4
+//////////////////////////////////////////////////////////////////////////
+
+    QCString trMemberFunctionDocumentationFortran()  
+    { return toUtf8(m_translator->trMemberFunctionDocumentationFortran()); }
+    QCString trCompoundListFortran()  
+    { return toUtf8(m_translator->trCompoundListFortran()); }
+    QCString trCompoundMembersFortran()  
+    { return toUtf8(m_translator->trCompoundMembersFortran()); }
+    QCString trCompoundListDescriptionFortran()  
+    { return toUtf8(m_translator->trCompoundListDescriptionFortran()); }
+    QCString trCompoundMembersDescriptionFortran(bool extractAll)  
+    { return toUtf8(m_translator->trCompoundMembersDescriptionFortran(extractAll)); }
+    QCString trCompoundIndexFortran()  
+    { return toUtf8(m_translator->trCompoundIndexFortran()); }
+    QCString trTypeDocumentation()  
+    { return toUtf8(m_translator->trTypeDocumentation()); }
+    QCString trSubprograms()  
+    { return toUtf8(m_translator->trSubprograms()); }
+    QCString trSubprogramDocumentation()  
+    { return toUtf8(m_translator->trSubprogramDocumentation()); }
+    QCString trDataTypes()  
+    { return toUtf8(m_translator->trDataTypes()); }
+    QCString trModulesList()  
+    { return toUtf8(m_translator->trModulesList()); }
+    QCString trModulesListDescription(bool extractAll)  
+    { return toUtf8(m_translator->trModulesListDescription(extractAll)); }
+    QCString trCompoundReferenceFortran(const char *clName,
+                                 ClassDef::CompoundType compType,
+                                 bool isTemplate)  
+    { return toUtf8(m_translator->trCompoundReferenceFortran(fromUtf8(clName),compType,isTemplate)); }
+    QCString trModuleReference(const char *namespaceName)  
+    { return toUtf8(m_translator->trModuleReference(fromUtf8(namespaceName))); }
+    QCString trModulesMembers()  
+    { return toUtf8(m_translator->trModulesMembers()); }
+    QCString trModulesMemberDescription(bool extractAll)  
+    { return toUtf8(m_translator->trModulesMemberDescription(extractAll)); }
+    QCString trModulesIndex()  
+    { return toUtf8(m_translator->trModulesIndex()); }
+    QCString trModule(bool first_capital, bool singular)  
+    { return toUtf8(m_translator->trModule(first_capital,singular)); }
+    QCString trGeneratedFromFilesFortran(ClassDef::CompoundType compType,
+                                          bool single)  
+    { return toUtf8(m_translator->trGeneratedFromFilesFortran(compType,single)); }
+    QCString trType(bool first_capital, bool singular)  
+    { return toUtf8(m_translator->trType(first_capital,singular)); }
+    QCString trSubprogram(bool first_capital, bool singular)  
+    { return toUtf8(m_translator->trSubprogram(first_capital,singular)); }
+    QCString trTypeConstraints()
+    { return toUtf8(m_translator->trTypeConstraints()); }
+
+
+//////////////////////////////////////////////////////////////////////////
   private:
     Translator *m_translator;
     void *m_toUtf8;
